import pytest

from django.contrib.auth.models import User
from django.core.cache import cache
from django.utils import timezone

from djpress.conf import settings as djpress_settings
from djpress.models import Post
from djpress.models.post import PUBLISHED_POSTS_CACHE_KEY


@pytest.mark.django_db
def test_get_cached_content(user, settings):
    # Confirm the settings in settings_testing.py
    assert djpress_settings.CACHE_RECENT_PUBLISHED_POSTS is False
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False

    # Create some test content
    Post.post_objects.create(
        title="Content 1",
        content="This is a test post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now(),
    )
    Post.post_objects.create(
        title="Content 2",
        content="This is another test post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now(),
    )

    # Call the _get_cached_recent_published_content method - this forces the cache to be set, regardless of settings
    queryset = Post.post_objects._get_cached_recent_published_posts()

    # Assert that the queryset is cached
    cached_queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset is not None
    assert len(queryset) == 2
    assert len(cached_queryset) == 2

    # Assert that subsequent calls retrieve the queryset from cache
    queryset2 = Post.post_objects._get_cached_recent_published_posts()
    assert list(queryset2) == list(cached_queryset)


@pytest.mark.django_db
def test_cache_invalidation_on_save(user):
    # Confirm the settings in settings_testing.py
    assert djpress_settings.CACHE_RECENT_PUBLISHED_POSTS is False

    # Create some test content
    content = Post.post_objects.create(
        title="Content 1",
        content="This is a test post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now(),
    )

    # Call the _get_cached_recent_published_content method - this forces the cache to be set, regardless of settings
    queryset = Post.post_objects._get_cached_recent_published_posts()

    # Assert that the queryset is cached
    cached_queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset is not None
    assert len(queryset) == 1

    # Modify the content and save it
    content.title = "Updated Content 1"
    content.save()

    # Assert that the cache is invalidated
    cached_queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset is None

    # Call the get_cached_published_content method again
    queryset2 = Post.post_objects._get_cached_recent_published_posts()

    # Assert that the queryset is cached again with the updated data
    cached_queryset2 = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset2 is not None
    assert len(queryset2) == 1
    assert queryset2[0].title == "Updated Content 1"


@pytest.mark.django_db
def test_cache_invalidation_on_delete(user):
    # Confirm the settings in settings_testing.py
    assert djpress_settings.CACHE_RECENT_PUBLISHED_POSTS is False

    # Create some test content
    content = Post.post_objects.create(
        title="Content 1",
        content="This is a test post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now(),
    )

    # Call the _get_cached_recent_published_content method - this forces the cache to be set, regardless of settings
    queryset = Post.post_objects._get_cached_recent_published_posts()

    # Assert that the queryset is cached
    cached_queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset is not None
    assert len(queryset) == 1

    # Delete the category
    content.delete()

    # Assert that the cache is invalidated
    cached_queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset is None

    # Call the get_cached_published_content method again
    queryset2 = Post.post_objects._get_cached_recent_published_posts()

    # Assert that the queryset is cached again with the updated data
    cached_queryset2 = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset2 is not None
    assert len(queryset2) == 0


@pytest.mark.django_db
def test_cache_get_recent_published_posts(user, settings):
    """Test that the get_recent_published_posts method returns the correct posts."""

    # Confirm settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    # Enable the posts cache
    settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] = True
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is True

    # Create some published posts
    post1 = Post.objects.create(title="Post 1", status="published", author=user, content="Test post")
    post2 = Post.objects.create(title="Post 2", status="published", author=user, content="Test post")
    post3 = Post.objects.create(title="Post 3", status="published", author=user, content="Test post")

    # Call the method being tested
    recent_posts = Post.post_objects.get_recent_published_posts()

    # Assert that the correct posts are returned
    assert list(recent_posts) == [post3, post2, post1]

    # Check that all posts are cached
    cached_queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset is not None
    assert list(cached_queryset) == [post3, post2, post1]

    # Test case 2: Limit the number of posts returned
    settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] = 2
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 2

    # Call the method being tested again
    recent_posts_2 = Post.post_objects.get_recent_published_posts()

    # # Assert that the correct posts are returned
    assert list(recent_posts_2) == [post3, post2]
    assert post1 not in recent_posts_2

    # Check that all posts are cached
    cached_queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert cached_queryset is not None
    assert list(cached_queryset) == [post3, post2]


@pytest.mark.django_db
def test_cache_get_recent_published_posts_future_post(user, settings):
    """Test that the get_recent_published_posts method returns the correct posts when there are future posts."""

    # Confirm settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    # Enable the posts cache
    settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] = True
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is True

    # Create some published posts
    post1 = Post.objects.create(title="Post 1", status="published", author=user, content="Test post")
    post2 = Post.objects.create(title="Post 2", status="published", author=user, content="Test post")

    # Call the method being tested
    recent_posts = Post.post_objects.get_recent_published_posts()

    # Assert that the correct posts are returned
    assert list(recent_posts) == [post2, post1]



# Generated by Django 5.1.3 on 2024-11-20 11:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0006_alter_post_parent"),
    ]

    operations = [
        migrations.CreateModel(
            name="PluginStorage",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("plugin_name", models.CharField(max_length=100, unique=True)),
                ("plugin_data", models.JSONField(default=dict)),
            ],
            options={
                "verbose_name": "plugin storage",
                "verbose_name_plural": "plugin storage",
            },
        ),
    ]



"""djpress admin configuration."""

from django.contrib import admin
from django.utils import timezone
from django.utils.html import format_html

# Register the models here.
from djpress.models import Category, PluginStorage, Post


@admin.register(Category)
class CategoryAdmin(admin.ModelAdmin):
    """Category admin configuration."""

    list_display = ["title", "slug"]


@admin.register(Post)
class PostAdmin(admin.ModelAdmin):
    """Post admin configuration."""

    list_display = ["post_type", "published_status", "title", "parent", "formatted_date", "author"]
    list_display_links = ["title"]
    ordering = ["post_type", "-date"]  # Displays pages first, then sorted by date.
    list_filter = ["post_type", "date", "author"]
    prepopulated_fields = {"slug": ("title",)}
    search_fields = ["title", "content", "slug"]
    readonly_fields = ["modified_date"]
    fieldsets = [
        (
            None,
            {
                "fields": ("title", "slug", "author"),
            },
        ),
        (
            "Content",
            {
                "fields": ("content",),
                "classes": ("wide",),
            },
        ),
        (
            "Publishing",
            {
                "fields": (
                    "status",
                    "post_type",
                    "date",
                    "modified_date",
                ),
            },
        ),
        (
            "Post Settings",
            {
                "fields": ("categories",),
                "description": "These settings only apply to posts",
            },
        ),
        (
            "Page Settings",
            {
                "fields": ("parent", "menu_order"),
                "description": "These settings only apply to pages",
            },
        ),
    ]

    @admin.display(
        boolean=True,
        description="Published",
    )
    def published_status(self, obj: Post) -> bool:
        """Display published status using Django's built-in boolean icon."""
        return obj.is_published

    def formatted_date(self, obj: Post) -> str:
        """Format the date and show future posts distinctly."""
        if obj.date > timezone.now():
            return format_html(
                '<span style="color: #666;">{} (Scheduled)</span>',
                obj.date.strftime("%Y-%m-%d %H:%M"),
            )
        return obj.date.strftime("%Y-%m-%d %H:%M")

    formatted_date.short_description = "Date"


@admin.register(PluginStorage)
class PluginStorageAdmin(admin.ModelAdmin):
    """PluginStorage admin configuration."""

    list_display = ["plugin_name", "plugin_data"]
    list_display_links = ["plugin_name"]
    ordering = ["plugin_name"]
    search_fields = ["plugin_name"]
    list_filter = ["plugin_name"]



import pytest

from django.contrib.auth.models import User

from djpress.conf import settings
from djpress.models import Category, Post
from djpress.templatetags.helpers import (
    categories_html,
    category_link,
    post_read_more_link,
    parse_post_wrapper_params,
)


@pytest.mark.django_db
def test_categories_html(category1, category2, category3):
    assert settings.CATEGORY_PREFIX == "test-url-category"

    categories = Category.objects.all()
    assert list(categories) == [category1, category2, category3]

    # Test case 1
    outer = "ul"
    outer_class = "categories"
    link_class = "category"
    expected_output = (
        '<ul class="categories">'
        f'<li><a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category {link_class}">{category1.title}</a></li>'
        f'<li><a href="/{settings.CATEGORY_PREFIX}/{category2.slug}/" title="View all posts in the {category2.title} category" class="p-category {link_class}">{category2.title}</a></li>'
        f'<li><a href="/{settings.CATEGORY_PREFIX}/{category3.slug}/" title="View all posts in the {category3.title} category" class="p-category {link_class}">{category3.title}</a></li>'
        "</ul>"
    )
    assert categories_html(categories, outer=outer, outer_class=outer_class, link_class=link_class) == expected_output

    # Test case 2
    outer = "ul"
    outer_class = ""
    link_class = ""
    expected_output = (
        "<ul>"
        f'<li><a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category">{category1.title}</a></li>'
        f'<li><a href="/{settings.CATEGORY_PREFIX}/{category2.slug}/" title="View all posts in the {category2.title} category" class="p-category">{category2.title}</a></li>'
        f'<li><a href="/{settings.CATEGORY_PREFIX}/{category3.slug}/" title="View all posts in the {category3.title} category" class="p-category">{category3.title}</a></li>'
        "</ul>"
    )
    assert categories_html(categories, outer=outer, outer_class=outer_class, link_class=link_class) == expected_output

    # Test case 3
    outer = "div"
    outer_class = "categories"
    link_class = "category"
    expected_output = (
        '<div class="categories">'
        f'<a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category {link_class}">{category1.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category2.slug}/" title="View all posts in the {category2.title} category" class="p-category {link_class}">{category2.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category3.slug}/" title="View all posts in the {category3.title} category" class="p-category {link_class}">{category3.title}</a>'
        "</div>"
    )
    assert categories_html(categories, outer=outer, outer_class=outer_class, link_class=link_class) == expected_output

    # Test case 4
    outer = "div"
    outer_class = ""
    link_class = ""
    expected_output = (
        "<div>"
        f'<a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category">{category1.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category2.slug}/" title="View all posts in the {category2.title} category" class="p-category">{category2.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category3.slug}/" title="View all posts in the {category3.title} category" class="p-category">{category3.title}</a>'
        "</div>"
    )
    assert categories_html(categories, outer=outer, outer_class=outer_class, link_class=link_class) == expected_output

    # Test case 5
    outer = "span"
    outer_class = "categories"
    link_class = "category"
    expected_output = (
        '<span class="categories">'
        f'<a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category {link_class}">{category1.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category2.slug}/" title="View all posts in the {category2.title} category" class="p-category {link_class}">{category2.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category3.slug}/" title="View all posts in the {category3.title} category" class="p-category {link_class}">{category3.title}</a>'
        "</span>"
    )
    assert categories_html(categories, outer=outer, outer_class=outer_class, link_class=link_class) == expected_output

    # Test case 6
    outer = "span"
    outer_class = ""
    link_class = ""
    expected_output = (
        "<span>"
        f'<a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category">{category1.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category2.slug}/" title="View all posts in the {category2.title} category" class="p-category">{category2.title}</a>, '
        f'<a href="/{settings.CATEGORY_PREFIX}/{category3.slug}/" title="View all posts in the {category3.title} category" class="p-category">{category3.title}</a>'
        "</span>"
    )
    assert categories_html(categories, outer=outer, outer_class=outer_class, link_class=link_class) == expected_output


@pytest.mark.django_db
def testcategory_link(category1):
    assert settings.CATEGORY_PREFIX == "test-url-category"

    # Test case 1 - no link class
    link_class = ""
    expected_output = f'<a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category">{category1.title}</a>'
    assert category_link(category1, link_class) == expected_output

    # Test case 2 - with link class
    link_class = "category-class"
    expected_output = f'<a href="/{settings.CATEGORY_PREFIX}/{category1.slug}/" title="View all posts in the {category1.title} category" class="p-category {link_class}">{category1.title}</a>'
    assert category_link(category1, link_class) == expected_output


@pytest.mark.django_db
def test_post_read_more_link(test_post1):
    assert settings.POST_READ_MORE_TEXT == "Test read more..."
    assert settings.POST_PREFIX == "test-posts"

    # Test case 1 - use the app settings for the read more text
    link_class = ""
    read_more_text = ""
    expected_output = f'<p><a href="{test_post1.url}">{settings.POST_READ_MORE_TEXT}</a></p>'
    assert post_read_more_link(test_post1, link_class, read_more_text) == expected_output

    # Test case 2 - use all options
    link_class = "read-more"
    read_more_text = "Continue reading"
    expected_output = f'<p><a href="{test_post1.url}" class="{link_class}">{read_more_text}</a></p>'
    assert post_read_more_link(test_post1, link_class, read_more_text) == expected_output


def test_parse_post_wrapper_params():
    # Test case 1
    params = ['tag="div"', 'class="blog-post"']
    expected_output = ("div", "blog-post")
    assert parse_post_wrapper_params(params) == expected_output

    # Test case 2
    params = ['class="blog-post"']
    expected_output = ("", "blog-post")
    assert parse_post_wrapper_params(params) == expected_output

    # Test case 3
    params = ['"div"', '"blog-post"']
    expected_output = ("div", "blog-post")
    assert parse_post_wrapper_params(params) == expected_output

    # Test case 4
    params = ['"blog-post"']
    expected_output = ("blog-post", "")
    assert parse_post_wrapper_params(params) == expected_output

    # Test case 5
    params = ['""', '"blog-post"']
    expected_output = ("", "blog-post")
    assert parse_post_wrapper_params(params) == expected_output

    # Test case 6
    params = ['tag="div"', 'class="blog-post"', 'extra="extra"']
    expected_output = ("div", "blog-post")
    assert parse_post_wrapper_params(params) == expected_output



# Generated by Django 5.1.2 on 2024-10-09 06:36

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0005_post_parent"),
    ]

    operations = [
        migrations.AlterField(
            model_name="post",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"post_type": "page"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="children",
                to="djpress.post",
            ),
        ),
    ]



"""Models package for djpress app."""

from djpress.models.category import Category
from djpress.models.plugin_storage import PluginStorage
from djpress.models.post import Post

__all__ = ["Category", "Post", "PluginStorage"]



"""Utility functions that are used in the project."""

from django.contrib.auth.models import User
from django.template.loader import TemplateDoesNotExist, select_template
from django.utils import timezone
from django.utils.module_loading import import_string

from djpress.conf import settings as djpress_settings


def get_markdown_renderer() -> callable:
    """Get the configured markdown renderer function.

    Returns:
        callable: The markdown renderer function.
    """
    renderer_path = djpress_settings.MARKDOWN_RENDERER

    try:
        return import_string(renderer_path)
    except ImportError as exc:
        from django.core.exceptions import ImproperlyConfigured

        msg = f"Could not import markdown renderer '{renderer_path}': {exc}"
        raise ImproperlyConfigured(msg) from exc


def get_author_display_name(user: User) -> str:
    """Return the author display name.

    Tries to display the first name and last name if available, otherwise falls back to
    the username.

    Args:
        user: The user.

    Returns:
        str: The author display name.
    """
    if user.first_name and user.last_name:
        return f"{user.first_name} {user.last_name}"

    if user.first_name:
        return user.first_name

    return user.username


def validate_date_parts(year: str | None, month: str | None, day: str | None) -> dict:
    """Validate the date parts.

    Args:
        year (str | None): The year.
        month (str | None): The month.
        day (str | None): The day.

    Returns:
        dict: The validated date parts.

    Raises:
        ValueError: If the date is invalid.
    """
    result = {}

    try:
        if year:
            result["year"] = int(year)
        if month:
            result["month"] = int(month)
        if day:
            result["day"] = int(day)

        # If we have all parts, try to create a date
        if "year" in result and "month" in result and "day" in result:
            timezone.make_aware(timezone.datetime(result["year"], result["month"], result["day"]))
        elif "year" in result and "month" in result:
            # Validate just year and month
            timezone.make_aware(timezone.datetime(result["year"], result["month"], 1))
        elif "year" in result:
            # Validate just the year
            timezone.make_aware(timezone.datetime(result["year"], 1, 1))

    except ValueError as exc:
        msg = "Invalid date"
        raise ValueError(msg) from exc

    return result


def get_templates(view_name: str) -> list[str]:
    """Get the template names for a view.

    Args:
        view_name (str): The view name.

    Returns:
        list[str]: The list of template names.
    """
    theme = djpress_settings.THEME

    template = ""

    if view_name == "index":
        template = f"djpress/{theme}/home.html"

    if view_name == "archive_posts":
        template = f"djpress/{theme}/archives.html"

    if view_name == "category_posts":
        template = f"djpress/{theme}/category.html"

    if view_name == "author_posts":
        template = f"djpress/{theme}/author.html"

    if view_name == "single_post":
        template = f"djpress/{theme}/single.html"

    if view_name == "single_page":
        template = f"djpress/{theme}/page.html"

    default_template = f"djpress/{theme}/index.html"

    return [template, default_template] if template else [default_template]


def get_template_name(view_name: str) -> str:
    """Return the first template that exists.

    Args:
        view_name (str): The view name

    Returns:
        str: The template name.
    """
    templates = get_templates(view_name)

    try:
        template = str(select_template(templates).template.name)
    except TemplateDoesNotExist as exc:
        msg = "No template found"
        raise TemplateDoesNotExist(msg) from exc

    return template



import pytest
from django.core.checks import Warning
from djpress.checks import check_plugin_hooks
from djpress.plugins import DJPressPlugin, registry, Hooks


def test_check_valid_hooks(clean_registry):
    """Test that check passes with valid hooks."""

    def test_callback(content: str) -> str:
        return content

    registry.register_hook(Hooks.PRE_RENDER_CONTENT, test_callback)
    registry.register_hook(Hooks.POST_RENDER_CONTENT, test_callback)

    warnings = check_plugin_hooks(None)
    assert len(warnings) == 0


def test_check_unknown_hook(clean_registry):
    """Test that check warns about unknown hook that isn't a Hooks enum."""

    def test_callback(content: str) -> str:
        return content

    # Directly manipulate registry to bypass type checking
    registry.hooks["unknown_hook"] = [test_callback]

    warnings = check_plugin_hooks(None)
    assert len(warnings) == 1
    warning = warnings[0]
    assert isinstance(warning, Warning)
    assert warning.id == "djpress.W001"
    assert "unknown hook 'unknown_hook'" in str(warning)
    assert "deprecated hook or a hook from a newer version" in warning.hint


def test_check_multiple_hooks(clean_registry):
    """Test check with mix of valid and invalid hooks."""

    def test_callback(content: str) -> str:
        return content

    # Register valid hooks normally
    registry.register_hook(Hooks.PRE_RENDER_CONTENT, test_callback)
    registry.register_hook(Hooks.POST_RENDER_CONTENT, test_callback)

    # Directly add invalid hook to bypass type checking
    registry.hooks["unknown_hook"] = [test_callback]

    warnings = check_plugin_hooks(None)
    assert len(warnings) == 1  # Only one warning for the unknown hook
    assert "unknown hook 'unknown_hook'" in str(warnings[0])


def test_check_detects_plugin_with_unknown_hook(bad_plugin_registry):
    """Test that check system detects plugin using unknown hook."""
    warnings = check_plugin_hooks(None)
    print(warnings)
    assert len(warnings) == 1
    assert "unknown hook 'foobar'" in str(warnings[0])


def test_check_detects_plugin_with_unknown_hook_not_loaded(clean_registry):
    """Test that check system detects plugin with unknown hook when plugins aren't loaded."""

    class BadPlugin(DJPressPlugin):
        name = "bad_plugin"

        def setup(self, registry):
            registry.register_hook("foobar", lambda x: x)

    # Manually create and setup the plugin
    plugin = BadPlugin()
    plugin.setup(registry)

    # Force registry to appear unloaded
    registry._loaded = False

    warnings = check_plugin_hooks(None)
    assert len(warnings) == 1
    assert "unknown hook 'foobar'" in str(warnings[0])


def test_check_loads_plugins_if_not_loaded(clean_registry):
    """Test that check loads plugins if they aren't already loaded."""
    from djpress.plugins import registry  # Import inside test

    # Reset registry state
    registry._loaded = False
    registry.plugins = []
    registry.hooks = {}

    # Mock load_plugins to verify it's called
    from unittest.mock import patch

    with patch("djpress.plugins.registry.load_plugins") as mock_load:
        check_plugin_hooks(None)
        mock_load.assert_called_once()


def test_check_skips_loading_plugins_if_already_loaded(clean_registry):
    """Test that check skips loading plugins if they're already loaded."""
    from djpress.plugins import registry

    # Make sure registry shows as loaded
    registry._loaded = True

    # Mock load_plugins to verify it's NOT called
    from unittest.mock import patch

    with patch("djpress.plugins.registry.load_plugins") as mock_load:
        check_plugin_hooks(None)
        mock_load.assert_not_called()



"""Djpress app configuration."""

from django.apps import AppConfig
from django.core.checks import Tags, register


class DjpressConfig(AppConfig):
    """Djpress app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "djpress"
    label = "djpress"

    def ready(self: "DjpressConfig") -> None:
        """Run when the app is ready."""
        # Import signals to ensure they are registered
        import djpress.signals  # noqa: F401

        # Initialize plugin system
        from djpress.plugins import registry

        registry.load_plugins()

        # Register check explicitly
        from djpress.checks import check_plugin_hooks

        register(check_plugin_hooks, Tags.compatibility)



"""Template tags for djpress."""

from django import template
from django.contrib.auth.models import User
from django.core.paginator import Page
from django.db import models
from django.template import Context
from django.urls import reverse
from django.utils.safestring import mark_safe

from djpress import url_utils
from djpress.conf import settings as djpress_settings
from djpress.exceptions import PageNotFoundError
from djpress.models import Category, Post
from djpress.templatetags import helpers
from djpress.utils import get_author_display_name

register = template.Library()


# Tags starting with `get_` are used to get data from the database.


@register.simple_tag(takes_context=True)
def get_recent_posts(context: Context) -> models.QuerySet[Post]:
    """Return the recent posts.

    This returns the most recent published posts, and tries to be efficient by checking if there's a `posts` object we
    can use.
    """
    posts: Page | None = context.get("posts")

    if isinstance(posts, Page) and posts.number == 1:
        return posts.object_list

    return Post.post_objects.get_recent_published_posts()


@register.simple_tag
def get_posts() -> models.QuerySet[Post]:
    """Return all published posts as a queryset.

    Returns:
        models.QuerySet[Post]: All posts.
    """
    return Post.post_objects.get_published_posts()


@register.simple_tag
def get_pages() -> models.QuerySet[Post]:
    """Return all published pages as a queryset.

    Returns:
        models.QuerySet[Post]: All pages.
    """
    return Post.page_objects.get_published_pages()


@register.simple_tag
def get_categories() -> models.QuerySet[Category] | None:
    """Return all categories as a queryset.

    Returns:
        models.QuerySet[Category]: All categories.
    """
    return Category.objects.get_categories().order_by("menu_order", "title")


@register.simple_tag
def blog_title() -> str:
    """Return the blog title.

    Returns:
        str: The blog title.
    """
    return djpress_settings.BLOG_TITLE


@register.simple_tag
def blog_title_link(link_class: str = "") -> str:
    """Return the blog title.

    Args:
        link_class: The CSS class(es) for the link.

    Returns:
        str: The blog title.
    """
    link_class_html = f' class="{link_class}"' if link_class else ""

    output = f'<a href="{reverse("djpress:index")}"{link_class_html}>{djpress_settings.BLOG_TITLE}</a>'

    return mark_safe(output)


@register.simple_tag
def blog_categories(
    outer: str = "ul",
    outer_class: str = "",
    link_class: str = "",
) -> str:
    """Return the categories of the blog.

    Args:
        outer: The outer HTML tag for the categories.
        outer_class: The CSS class(es) for the outer tag.
        link_class: The CSS class(es) for the link.

    Returns:
        str: The categories of the blog.
    """
    categories = Category.objects.all()
    if not categories:
        return ""

    return mark_safe(helpers.categories_html(categories, outer, outer_class, link_class))


@register.simple_tag
def blog_pages_list(ul_outer_class: str = "", li_class: str = "", a_class: str = "", ul_child_class: str = "") -> str:
    """Returns an HTML list of the blog pages.

    The pages are sorted by menu order and then by title. Pages that have children have a nested list of children.

    The default output with no arguments is an unordered list with no classes. CSS classes can be added to the output by
    specifying the arguments shown below.

    ```
    <ul class="ul_outer_class">
        <li class="li_class">
            <a href="/page1/" class="a_class">Page 1</a>
        </li>
        <li class="li_class">
            <a href="/page2/" class="a_class">Page 2</a>
        </li>
        <li class="li_parent_class">
            <a href="/page3/" class="a_class">Page 3</a>
            <ul class="ul_child_class">
                <li class="li_class">
                    <a href="/page3/child1/" class="a_class">Child 1</a>
                </li>
                <li class="li_class">
                    <a href="/page3/child2/" class="a_class">Child 2</a>
                </li>
            </ul>
        </li>
    </ul>
    ```

    Args:
        ul_outer_class (str): The CSS class(es) for the outer unordered list.
        li_class (str): The CSS class(es) for the
        a_class (str): The CSS class(es) for the anchor tags.
        ul_child_class (str): The CSS class(es) for the nested unordered lists.

    Returns:
        str: The HTML list of the blog pages.
    """
    pages = Post.page_objects.get_page_tree()

    output = ""

    if not pages:
        return output

    if ul_outer_class:
        ul_outer_class = f' class="{ul_outer_class}"'

    output += f"<ul{ul_outer_class}>"
    output += helpers.get_blog_pages_list(pages, li_class=li_class, a_class=a_class, ul_child_class=ul_child_class)
    output += "</ul>"

    return mark_safe(output)


@register.simple_tag
def blog_pages(
    outer: str = "ul",
    outer_class: str = "",
    link_class: str = "",
) -> str:
    """Return the pages of the blog.

    Args:
        outer: The outer HTML tag for the pages.
        outer_class: The CSS class(es) for the outer tag.
        link_class: The CSS class(es) for the link.

    Returns:
        str: The pages of the blog.
    """
    pages = Post.page_objects.get_published_pages()

    if not pages:
        return ""

    output = ""

    outer_class_html = f' class="{outer_class}"' if outer_class else ""

    if outer == "ul":
        output += f"<ul{outer_class_html}>"
        for page in pages:
            output += f"<li>{helpers.get_page_link(page=page, link_class=link_class)}</li>"
        output += "</ul>"

    if outer == "div":
        output += f"<div{outer_class_html}>"
        for page in pages:
            output += f"{helpers.get_page_link(page=page, link_class=link_class)}, "
        output = output[:-2]  # Remove the trailing comma and space
        output += "</div>"

    if outer == "span":
        output += f"<span{outer_class_html}>"
        for page in pages:
            output += f"{helpers.get_page_link(page=page, link_class=link_class)}, "
        output = output[:-2]  # Remove the trailing comma and space
        output += "</span>"

    return mark_safe(output)


@register.simple_tag(takes_context=True)
def blog_page_title(
    context: Context,
    pre_text: str = "",
    post_text: str = "",
) -> str:
    """Return the page title.

    Args:
        context: The context.
        pre_text: The text to prepend to the page title.
        post_text: The text to append to the page title.

    Returns:
        str: The page title.
    """
    category: Category | None = context.get("category")
    author: User | None = context.get("author")
    post: Post | None = context.get("post")

    if category:
        page_title = category.title

    elif author:
        page_title = get_author_display_name(author)

    elif post:
        page_title = post.title
    else:
        page_title = ""

    if page_title:
        page_title = f"{pre_text}{page_title}{post_text}"

    return page_title


@register.simple_tag(takes_context=True)
def have_posts(context: Context) -> list[Post | None] | Page:
    """Return the posts in the context.

    If there's a `post` in the context, then we return a list with that post.

    If there's a `posts` in the context, then we return the posts. The `posts` should
    be a Page object.

    Args:
        context: The context.

    Returns:
        list[Post]: The posts in the context.
    """
    post: Post | None = context.get("post")
    posts: Page | None = context.get("posts")

    if post:
        return [post]
    if posts:
        return posts

    return []


@register.simple_tag(takes_context=True)
def get_post_title(context: Context) -> str:
    """Return the title of a post.

    This is just the title of the post from the current context with no further HTML.

    Args:
        context: The context.

    Returns:
        str: The title of the post.
    """
    post: Post | None = context.get("post")
    if not post:
        return ""

    return post.title


@register.simple_tag(takes_context=True)
def post_title(context: Context, *, outer_tag: str = "", link_class: str = "", force_link: bool = False) -> str:
    """Return the title link for a post.

    If the post is part of a posts collection, then return the title and a link to the post. If the post is a single
    post, then return just the title of the post with no link. But this behavior can be overridden by setting
    `force_link` to `True`.

    The outer tag can be any of the following: "h1", "h2", "h3", "h4", "h5", "h6", "p", "div", "span". If the outer tag
    is not one of these, then the title will be returned with no outer tag.

    If the outer tag is one of the allowed tags, and if Microformats are enabled, then the outer tag will have the class
    "p-name".

    Otherwise return an empty string.

    Args:
        context: The context.
        outer_tag: The outer HTML tag for the title.
        link_class: The CSS class(es) for the link.
        force_link: Whether to force the link to be displayed.

    Returns:
        str: The title link for the post.
    """
    post: Post | None = context.get("post")
    posts: Page | None = context.get("posts")

    # If there's no post in the context, return an empty string.
    if not post:
        return ""

    # Get the title of the post
    output = post.title

    # If there's a posts in the context, or if the link is forced, then we need to display the link.
    if posts or force_link:
        # Build the classes for the link
        link_classes = ""

        # Add p-category if microformats are enabled
        if djpress_settings.MICROFORMATS_ENABLED:
            link_classes += "u-url "

        # Add the user-defined link class
        link_classes += link_class

        # Trim any trailing spaces
        link_classes = link_classes.strip()

        link_class_html = f' class="{link_classes}"' if link_classes else ""

        output = f'<a href="{post.url}" title="{output}"{link_class_html}>{output}</a>'

    # If the outer tag is one of the allowed tags, then wrap the output in the outer tag.
    if outer_tag in ["h1", "h2", "h3", "h4", "h5", "h6", "p", "div", "span"]:
        # If Microformats are enabled, use p-name with the outer tag.
        mf = ' class="p-name"' if djpress_settings.MICROFORMATS_ENABLED else ""

        output = f"<{outer_tag}{mf}>{output}</{outer_tag}>"

    return mark_safe(output)


@register.simple_tag(takes_context=True)
def get_post_author(context: Context) -> str:
    """Return the author display name.

    Tries to display the first name and last name if available, otherwise falls back to
    the username.

    Args:
        context: The context.

    Returns:
        str: The author display name.
    """
    post: Post | None = context.get("post")
    if not post:
        return ""

    author = post.author

    return get_author_display_name(author)


@register.simple_tag(takes_context=True)
def post_author(context: Context, link_class: str = "") -> str:
    """Return the author link for a post.

    Args:
        context: The context.
        link_class: The CSS class(es) for the link.

    Returns:
        str: The author link.
    """
    post: Post | None = context.get("post")
    if not post:
        return ""

    author = post.author
    author_display_name = get_author_display_name(author)

    mf = ' class="p-author"' if djpress_settings.MICROFORMATS_ENABLED else ""

    author_html = f"<span{mf}>{author_display_name}</span>"

    if not djpress_settings.AUTHOR_ENABLED:
        return author_html

    author_url = url_utils.get_author_url(user=author)

    link_class_html = f' class="{link_class}"' if link_class else ""

    output = (
        f'<a href="{author_url}" title="View all posts by '
        f'{author_display_name}"{link_class_html}>'
        f"{author_html}</a>"
    )

    return mark_safe(output)


@register.simple_tag
def post_category_link(category: Category, link_class: str = "") -> str:
    """Return the category links for a post.

    TODO: do we need this?

    Args:
        category: The category of the post.
        link_class: The CSS class(es) for the link.
    """
    if not djpress_settings.CATEGORY_ENABLED:
        return category.title

    return mark_safe(helpers.category_link(category, link_class))


@register.simple_tag(takes_context=True)
def get_post_date(context: Context) -> str:
    """Return the date of a post.

    Args:
        context: The context.

    Returns:
        str: The date of the post.
    """
    post: Post | None = context.get("post")
    if not post:
        return ""

    output_date = post.date
    return output_date.strftime("%b %-d, %Y")


@register.simple_tag(takes_context=True)
def post_date(context: Context, link_class: str = "") -> str:
    """Return the date link for a post.

    Args:
        context: The context.
        link_class: The CSS class(es) for the link.

    Returns:
        str: The date link for the post.
    """
    post: Post | None = context.get("post")
    if not post:
        return ""
    output_date = post.date

    if not djpress_settings.ARCHIVE_ENABLED:
        return mark_safe(output_date.strftime("%b %-d, %Y"))

    post_year = output_date.strftime("%Y")
    post_month = output_date.strftime("%m")
    post_month_name = output_date.strftime("%b")
    post_day = output_date.strftime("%d")
    post_day_name = output_date.strftime("%-d")
    post_time = output_date.strftime("%-I:%M %p")

    year_url = url_utils.get_archives_url(year=int(post_year))
    month_url = url_utils.get_archives_url(year=int(post_year), month=int(post_month))
    day_url = url_utils.get_archives_url(year=int(post_year), month=int(post_month), day=int(post_day))

    link_class_html = f' class="{link_class}"' if link_class else ""

    output = (
        f'<a href="{month_url}" title="View all posts in {post_month_name} {post_year}"'
        f"{link_class_html}>{post_month_name}</a> "
        f'<a href="{day_url}" title="View all posts on {post_day_name} '
        f'{post_month_name} {post_year}"{link_class_html}>{post_day_name}</a>, '
        f'<a href="{year_url}" title="View all posts in {post_year}"{link_class_html}>'
        f"{post_year}</a>, "
        f"{post_time}."
    )

    # If Microformats are enabled, use dt-published with the date.
    if djpress_settings.MICROFORMATS_ENABLED:
        output = f'<time class="dt-published" datetime="{output_date.isoformat()}">{output}</time>'

    return mark_safe(output)


@register.simple_tag(takes_context=True)
def post_content(
    context: Context,
    *,
    outer_tag: str = "",
    read_more_link_class: str = "",
    read_more_text: str = "",
) -> str:
    """Return the content of a post.

    If the post is part of a posts collection, then we return the truncated content of
    the post with the read more link.

    If the post is a single post, return the full content of the post.

    The outer tag can be any one of the following: "section", "div", "article", "p", "span". If the outer tag is not one
    of these, then the content will be returned with no outer tag.

    If there's an outer tag, and if microformats are enabled, then the outer tag will have the class "e-content".

    If there's no post, return an empty string.

    Args:
        context: The context.
        outer_tag: The outer HTML tag for the content.
        read_more_link_class: The CSS class(es) for the read more link.
        read_more_text: The text for the read more link.

    Returns:
        str: The content of the post.
    """
    # Check if there's a post or posts in the context.
    post: Post | None = context.get("post")
    posts: Page | None = context.get("posts")

    # If there's no post, return an empty string.
    if not post:
        return ""

    # If there's a posts in the context, then we need to display the truncated content.
    if posts:
        content = post.truncated_content_markdown
        if post.is_truncated:
            content += helpers.post_read_more_link(post, read_more_link_class, read_more_text)
    else:
        content = post.content_markdown

    # If the outer tag is one of the allowed tags, then wrap the output in the outer tag.
    if outer_tag in ["section", "div", "article", "p", "span"]:
        # If Microformats are enabled, use e-content with the outer tag.
        mf = ' class="e-content"' if djpress_settings.MICROFORMATS_ENABLED else ""

        content = f"<{outer_tag}{mf}>{content}</{outer_tag}>"

    return mark_safe(content)


@register.simple_tag(takes_context=True)
def category_title(
    context: Context,
    outer: str = "",
    outer_class: str = "",
    pre_text: str = "",
    post_text: str = "",
) -> str:
    """Return the title of a category.

    Expects there to be an `category` in the context set to a Category object. If
    there's no category in the context or category is not a Category object, then return
    an empty string.


    Args:
        context: The context.
        outer: The outer HTML tag for the category.
        outer_class: The CSS class(es) for the outer tag.
        pre_text: The text to prepend to the category title.
        post_text: The text to append to the category title.

    Returns:
        str: The title of the category formatted with the outer tag and class if
        provided.
    """
    category: Category | None = context.get("category")

    if not category or not isinstance(category, Category):
        return ""

    allowed_outer_tags = ["h1", "h2", "h3", "h4", "h5", "h6", "p", "div", "span"]
    outer_class = f' class="{outer_class}"' if outer_class else ""

    output = category.title

    if pre_text:
        output = f"{pre_text}{output}"

    if post_text:
        output = f"{output}{post_text}"

    if outer in allowed_outer_tags:
        return mark_safe(f"<{outer}{outer_class}>{output}</{outer}>")

    return output


@register.simple_tag(takes_context=True)
def author_name(
    context: Context,
    outer: str = "",
    outer_class: str = "",
    pre_text: str = "",
    post_text: str = "",
) -> str:
    """Return the name of an author.

    Expects there to be an `author` in the context set to a user object. If there's no
    author in the context or author is not a User object, then return an empty string.

    Args:
        context: The context.
        outer: The outer HTML tag for the author.
        outer_class: The CSS class(es) for the outer tag.
        pre_text: The text to prepend to the author name.
        post_text: The text to append to the author name.

    Returns:
        str: The name of the author formatted with the outer tag and class if
        provided.
    """
    author: User | None = context.get("author")

    if not author or not isinstance(author, User):
        return ""

    allowed_outer_tags = ["h1", "h2", "h3", "h4", "h5", "h6", "p", "div", "span"]
    outer_class = f' class="{outer_class}"' if outer_class else ""

    output = get_author_display_name(author)

    if pre_text:
        output = f"{pre_text}{output}"

    if post_text:
        output = f"{output}{post_text}"

    if outer in allowed_outer_tags:
        return mark_safe(f"<{outer}{outer_class}>{output}</{outer}>")

    return output


@register.simple_tag(takes_context=True)
def post_categories(
    context: Context,
    outer: str = "ul",
    outer_class: str = "",
    link_class: str = "",
) -> str:
    """Return the categories of a post.

    Each category is a link to the category page.

    Args:
        context: The context.
        outer: The outer HTML tag for the categories.
        outer_class: The CSS class(es) for the outer tag.
        link_class: The CSS class(es) for the link.

    Returns:
        str: The categories of the post.
    """
    post: Post | None = context.get("post")
    if not post:
        return ""

    categories = post.categories.all()
    if not categories:
        return ""

    return mark_safe(helpers.categories_html(categories, outer, outer_class, link_class))


@register.simple_tag(takes_context=True)
def is_paginated(context: Context) -> bool:
    """Return whether the posts are paginated.

    Args:
        context: The context.

    Returns:
        bool: Whether the posts are paginated.
    """
    posts: Page | None = context.get("posts")
    if not posts or not isinstance(posts, Page):  # noqa: SIM103
        return False

    return True


@register.simple_tag(takes_context=True)
def get_pagination_range(context: Context) -> range:
    """Return the range of pagination pages.

    Args:
        context: The context.

    Returns:
        range: The pagination pages.
    """
    page: Page | None = context.get("posts")
    if not page or not isinstance(page, Page):
        return range(0)

    return page.paginator.page_range


@register.simple_tag(takes_context=True)
def get_pagination_current_page(context: Context) -> int:
    """Return the current page number.

    Args:
        context: The context.

    Returns:
        int: The current page number.
    """
    page: Page | None = context.get("posts")
    if not page or not isinstance(page, Page):
        return 0

    return page.number


@register.simple_tag(takes_context=True)
def pagination_links(
    context: Context,
) -> str:
    """Return the previous and next post links.

    This checks if there is a Page object in the context. If there is, then we return
    the previous and next post links. If there is no Page object in the context, then
    we return an empty string.

    Args:
        context: The context.

    Returns:
        str: The previous and next post links.
    """
    page: Page | None = context.get("posts")
    if not page or not isinstance(page, Page):
        return ""

    if page.has_previous():
        previous_output = (
            f'<span class="previous">'
            f'<a href="?page=1">&laquo; first</a> '
            f'<a href="?page={page.previous_page_number()}">previous</a>'
            f"</span>"
        )
    else:
        previous_output = ""

    if page.has_next():
        next_output = (
            f'<span class="next">'
            f'<a href="?page={page.next_page_number()}">next</a> '
            f'<a href="?page={page.paginator.num_pages}">last &raquo;</a>'
            f"</span>"
        )
    else:
        next_output = ""

    current_output = f'<span class="current">Page {page.number} of {page.paginator.num_pages}</span>'

    return mark_safe(
        f'<div class="pagination">{previous_output} {current_output} {next_output}</div>',
    )


@register.simple_tag()
def page_link(
    page_slug: str,
    outer: str = "div",
    outer_class: str = "",
    link_class: str = "",
) -> str:
    """Return the link to a page.

    Args:
        page_slug: The slug of the page.
        outer: The outer HTML tag for the page link.
        outer_class: The CSS class(es) for the outer tag.
        link_class: The CSS class(es) for the link.

    Returns:
        str: The link to the page.
    """
    try:
        page: Post | None = Post.page_objects.get_published_page_by_slug(page_slug)
    except PageNotFoundError:
        return ""

    output = helpers.get_page_link(page, link_class=link_class)

    if outer == "li":
        return mark_safe(f"<li{outer_class}>{output}</li>")

    if outer == "span":
        return mark_safe(f"<span{outer_class}>{output}</span>")

    if outer == "div":
        return mark_safe(f"<div{outer_class}>{output}</div>")

    return mark_safe(output)


@register.simple_tag
def get_rss_url() -> str:
    """Return the URL to the RSS feed.

    Returns:
        str: The URL to the RSS feed.
    """
    return url_utils.get_rss_url()


@register.simple_tag
def rss_link() -> str:
    """Return an HTML link to the RSS feed.

    If the RSS feed is enabled, then return an HTML link tag that points to the RSS feed. Otherwise, return an empty
    string.

    Example:
        ```django
        {% rss_link %}
        ```

        ```html
        <link rel="alternate" type="application/rss+xml" title="Latest Posts" href="/rss/">
        ```

    Returns:
        str: The HTML link to the RSS feed.
    """
    if not djpress_settings.RSS_ENABLED:
        return ""

    rss_url = url_utils.get_rss_url()

    return mark_safe(f'<link rel="alternate" type="application/rss+xml" title="Latest Posts" href="{rss_url}">')


@register.tag(name="post_wrap")
def post_wrapper_tag(parser: template.base.Parser, token: template.base.Token) -> helpers.BlogPostWrapper:
    """Parse the blog post wrapper tag.

    This is a template tag that wraps the blog post content in a configurable HTML tag with a CSS class.

    Example usage:
        {% post_wrap "article" "post" %}<p>Post content</p>{% end_post_wrap %}

    Args:
        parser: The template parser.
        token: The template token.

    Returns:
        BlogPostWrapper: The blog post wrapper tag.
    """
    params = token.split_contents()[1:]  # skip the tag name

    tag, css_class = helpers.parse_post_wrapper_params(params)

    # If microformats are enabled, add the h-entry class before the css class
    if djpress_settings.MICROFORMATS_ENABLED:
        css_class = f"h-entry {css_class}" if css_class else "h-entry"

    if css_class:
        css_class = f' class="{css_class}"'

    nodelist = parser.parse(("end_post_wrap",))
    parser.delete_first_token()

    return helpers.BlogPostWrapper(nodelist, tag, css_class)



# Generated by Django 5.0.4 on 2024-05-02 10:46

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("slug", models.SlugField(blank=True, unique=True)),
                ("description", models.TextField(blank=True)),
            ],
            options={
                "verbose_name": "category",
                "verbose_name_plural": "categories",
            },
        ),
        migrations.CreateModel(
            name="Post",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                ("slug", models.SlugField(blank=True, unique=True)),
                ("content", models.TextField()),
                ("date", models.DateTimeField(default=django.utils.timezone.now)),
                ("modified_date", models.DateTimeField(auto_now=True)),
                (
                    "status",
                    models.CharField(
                        choices=[("draft", "Draft"), ("published", "Published")],
                        default="draft",
                        max_length=10,
                    ),
                ),
                (
                    "content_type",
                    models.CharField(
                        choices=[("post", "Post"), ("page", "Page")],
                        default="post",
                        max_length=10,
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "categories",
                    models.ManyToManyField(blank=True, to="djpress.category"),
                ),
            ],
            options={
                "verbose_name": "post",
                "verbose_name_plural": "posts",
            },
        ),
    ]



"""Sitemap classes for DJ Press."""

from datetime import datetime
from typing import Any

from django.contrib.sitemaps import Sitemap
from django.db.models import QuerySet

from djpress.conf import settings as djpress_settings
from djpress.models import Category, Post


class PostSitemap(Sitemap):
    """Sitemap for blog posts."""

    changefreq = "monthly"
    protocol = "https"

    def items(self) -> QuerySet:
        """Return all published posts."""
        return Post.post_objects.all()

    def lastmod(self, obj: Post) -> datetime:
        """Return the last modified date of the post."""
        return obj.modified_date

    def location(self, obj: Post) -> str:
        """Return the URL of the post."""
        return obj.url


class PageSitemap(Sitemap):
    """Sitemap for static pages."""

    changefreq = "monthly"
    protocol = "https"

    def items(self) -> QuerySet:
        """Return all published pages."""
        return Post.page_objects.get_published_pages()

    def lastmod(self, obj: Post) -> datetime:
        """Return the last modified date of the page."""
        return obj.modified_date

    def location(self, obj: Post) -> str:
        """Return the URL of the page."""
        return obj.url


class CategorySitemap(Sitemap):
    """Sitemap for category archives."""

    changefreq = "daily"
    protocol = "https"

    def items(self) -> QuerySet:
        """Return all categories that have published posts."""
        return Category.objects.get_categories_with_published_posts()

    def lastmod(self, obj: Category) -> datetime | None:
        """Return the last modified date of the most recent post in the category."""
        return obj.last_modified

    def location(self, obj: Category) -> str:
        """Return the URL of the category."""
        return obj.url


class DateBasedSitemap(Sitemap):
    """Sitemap for date-based archives."""

    changefreq = "daily"
    protocol = "https"

    def items(self) -> list[dict[str, Any]]:
        """Return all date-based archives that have posts.

        Returns a list of dictionaries containing:
        - year: The year
        - month: The month (optional)
        - day: The day (optional)
        - latest_modified: The latest modified date of posts in this period
        """
        if djpress_settings.ARCHIVE_ENABLED is False:
            return []

        archives: list[dict] = []

        # Get all unique years
        years = Post.post_objects.get_years()
        for year in years:
            archives.append(
                {
                    "year": year.year,
                    "latest_modified": Post.post_objects.get_year_last_modified(year.year),
                },
            )

            # Get all months in this year
            months = Post.post_objects.get_months(year.year)
            for month in months:
                archives.append(
                    {
                        "year": year.year,
                        "month": month.month,
                        "latest_modified": Post.post_objects.get_month_last_modified(year.year, month.month),
                    },
                )

                # Get all days in this month
                days = Post.post_objects.get_days(year.year, month.month)
                for day in days:
                    archives.append(  # noqa: PERF401
                        {
                            "year": year.year,
                            "month": month.month,
                            "day": day.day,
                            "latest_modified": Post.post_objects.get_day_last_modified(year.year, month.month, day.day),
                        },
                    )

        return archives

    def lastmod(self, obj: dict[str, Any]) -> datetime:
        """Return the last modified date of posts in this archive."""
        return obj["latest_modified"]

    def location(self, obj: dict[str, Any]) -> str:
        """Return the URL of the archive."""
        from djpress.url_utils import get_archives_url

        return get_archives_url(obj["year"], obj.get("month"), obj.get("day"))



# test_category_cache.py

import pytest

from django.core.cache import cache

from djpress.conf import settings
from djpress.models import Category
from djpress.models.category import CATEGORY_CACHE_KEY


@pytest.mark.django_db
def test_get_cached_categories():
    # Confirm the settings in settings_testing.py
    assert settings.CACHE_CATEGORIES is True

    # Create some test categories
    Category.objects.create(title="Category 1")
    Category.objects.create(title="Category 2")

    # Call the get_cached_categories method
    queryset = Category.objects._get_cached_categories()

    # Assert that the queryset is cached
    cached_queryset = cache.get(CATEGORY_CACHE_KEY)
    assert cached_queryset is not None
    assert len(queryset) == 2
    assert len(cached_queryset) == 2

    # Assert that subsequent calls retrieve the queryset from cache
    queryset2 = Category.objects._get_cached_categories()
    assert list(queryset2) == list(cached_queryset)


@pytest.mark.django_db
def test_cache_invalidation_on_save():
    # Confirm the settings in settings_testing.py
    assert settings.CACHE_CATEGORIES is True

    # Create a test category
    category = Category.objects.create(title="Category 1")

    # Call the get_cached_categories method
    queryset = Category.objects._get_cached_categories()

    # Assert that the queryset is cached
    cached_queryset = cache.get(CATEGORY_CACHE_KEY)
    assert cached_queryset is not None
    assert len(queryset) == 1

    # Modify the category and save it
    category.title = "Updated Category"
    category.save()

    # Assert that the cache is invalidated
    cached_queryset = cache.get(CATEGORY_CACHE_KEY)
    assert cached_queryset is None

    # Call the get_cached_categories method again
    queryset2 = Category.objects._get_cached_categories()

    # Assert that the queryset is cached again with the updated data
    cached_queryset2 = cache.get(CATEGORY_CACHE_KEY)
    assert cached_queryset2 is not None
    assert len(queryset2) == 1
    assert queryset2[0].title == "Updated Category"


@pytest.mark.django_db
def test_cache_invalidation_on_delete():
    # Confirm the settings in settings_testing.py
    assert settings.CACHE_CATEGORIES is True

    # Create a test category
    category = Category.objects.create(title="Category 1")

    # Call the get_cached_categories method
    queryset = Category.objects._get_cached_categories()

    # Assert that the queryset is cached
    cached_queryset = cache.get(CATEGORY_CACHE_KEY)
    assert cached_queryset is not None
    assert len(queryset) == 1

    # Delete the category
    category.delete()

    # Assert that the cache is invalidated
    cached_queryset = cache.get(CATEGORY_CACHE_KEY)
    assert cached_queryset is None

    # Call the get_cached_categories method again
    queryset2 = Category.objects._get_cached_categories()

    # Assert that the queryset is cached again with the updated data
    cached_queryset2 = cache.get(CATEGORY_CACHE_KEY)
    assert cached_queryset2 is not None
    assert len(queryset2) == 0


@pytest.mark.django_db
def test_cache_get_category_by_slug():
    """Test that the get_category_by_slug method returns the correct category."""
    # Confirm the settings in settings_testing.py
    assert settings.CACHE_CATEGORIES is True

    category1 = Category.objects.create(title="Category 1", slug="category-1")
    category2 = Category.objects.create(title="Category 2", slug="category-2")

    category = Category.objects.get_category_by_slug("category-1")

    assert category == category1
    assert not category == category2


@pytest.mark.django_db
def test_cache_get_category_by_slug_not_in_cache():
    """Test that the get_category_by_slug method returns the correct category."""
    # Confirm the settings in settings_testing.py
    assert settings.CACHE_CATEGORIES is True

    category1 = Category.objects.create(title="Category 1", slug="category-1")
    category2 = Category.objects.create(title="Category 2", slug="category-2")

    # Call the get_cached_categories method
    Category.objects._get_cached_categories()

    cached_queryset = cache.get(CATEGORY_CACHE_KEY)

    assert cached_queryset is not None
    assert category1 in cached_queryset
    assert category2 in cached_queryset

    cache.delete(CATEGORY_CACHE_KEY)
    cached_queryset = cache.get(CATEGORY_CACHE_KEY)

    assert cached_queryset is None

    category = Category.objects.get_category_by_slug("category-1")

    assert category == category1
    assert not category == category2


@pytest.mark.django_db
def test_cache_get_category_by_slug_not_exists():
    """Test that the get_category_by_slug method returns None when the category does not exist."""
    # Confirm the settings in settings_testing.py
    assert settings.CACHE_CATEGORIES is True

    with pytest.raises(ValueError) as excinfo:
        _ = Category.objects.get_category_by_slug("non-existent-category")
    assert "Category not found" in str(excinfo.value)



import pytest
from django.utils import timezone
from unittest.mock import Mock
from django.core.cache import cache
from django.core.exceptions import ValidationError
from django.db.models import QuerySet

from djpress.models import Category, Post
from djpress import urls as djpress_urls
from djpress.models.post import PUBLISHED_POSTS_CACHE_KEY
from djpress.exceptions import PostNotFoundError, PageNotFoundError


@pytest.mark.django_db
def test_post_model(test_post1, user, category1):
    test_post1.categories.add(category1)
    assert test_post1.title == "Test Post1"
    assert test_post1.slug == "test-post1"
    assert test_post1.author == user
    assert test_post1.status == "published"
    assert test_post1.post_type == "post"
    assert test_post1.categories.count() == 1
    assert str(test_post1) == "Test Post1"


@pytest.mark.django_db
def test_post_default_queryset(test_post1, test_post2, test_post3):
    """Make sure the default queryset returns only published posts."""
    assert list(Post.objects.all()) == [test_post1, test_post2, test_post3]

    test_post1.status = "draft"
    test_post1.save()
    assert list(Post.objects.all()) == [test_post2, test_post3]

    test_post2.date = timezone.now() + timezone.timedelta(days=1)
    test_post2.save()
    assert list(Post.objects.all()) == [test_post3]


@pytest.mark.django_db
def test_get_published_content_with_future_date(user):
    Post.post_objects.create(
        title="Past Post",
        slug="past-post",
        content="This is a past post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now() - timezone.timedelta(days=1),
    )
    Post.post_objects.create(
        title="Future Post",
        slug="future-post",
        content="This is a future post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now() + timezone.timedelta(days=1),
    )
    assert Post.admin_objects.all().count() == 2
    assert Post.post_objects.get_published_posts().count() == 1


@pytest.mark.django_db
def test_get_published_content_ordering(user):
    Post.post_objects.create(
        title="Older Post",
        slug="older-post",
        content="This is an older post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now() - timezone.timedelta(days=2),
    )
    Post.post_objects.create(
        title="Newer Post",
        slug="newer-post",
        content="This is a newer post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now() - timezone.timedelta(days=1),
    )
    posts = Post.post_objects.all()
    assert posts[0].title == "Newer Post"
    assert posts[1].title == "Older Post"


@pytest.mark.django_db
def test_get_published_post_by_slug_with_future_date(user):
    Post.post_objects.create(
        title="Future Post",
        slug="future-post",
        content="This is a future post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now() + timezone.timedelta(days=1),
    )
    with pytest.raises(PostNotFoundError):
        Post.post_objects.get_published_post_by_slug("future-post")


@pytest.mark.django_db
def test_get_published_content_by_category_with_future_date(user):
    category = Category.objects.create(title="Test Category", slug="test-category")
    Post.post_objects.create(
        title="Past Post",
        slug="past-post",
        content="This is a past post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now() - timezone.timedelta(days=1),
    ).categories.add(category)
    Post.post_objects.create(
        title="Future Post",
        slug="future-post",
        content="This is a future post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.now() + timezone.timedelta(days=1),
    ).categories.add(category)
    assert Post.post_objects.get_published_posts_by_category(category).count() == 1


@pytest.mark.django_db
def test_post_slug_generation(user):
    # Test case 1: Slug generated from title
    post1 = Post.post_objects.create(
        title="My First Blog Post",
        content="This is the content of my first blog post.",
        author=user,
    )
    assert post1.slug == "my-first-blog-post"

    # Test case 2: Slug not overridden when provided
    post2 = Post.post_objects.create(
        title="My Second Blog Post",
        slug="custom-slug",
        content="This is the content of my second blog post.",
        author=user,
    )
    assert post2.slug == "custom-slug"

    # Test case 3: Slug generated with special characters
    post3 = Post.post_objects.create(
        title="My Third Blog Post!",
        content="This is the content of my third blog post.",
        author=user,
    )
    assert post3.slug == "my-third-blog-post"

    # Test case 4: Slug generated with non-ASCII characters
    post4 = Post.post_objects.create(
        title="My Post with 😊 Emoji",
        content="This is the content of the post with an emoji in the title.",
        author=user,
    )
    assert post4.slug == "my-post-with-emoji"

    # Test case 5: Raise error for invalid title
    with pytest.raises(ValueError) as exc_info:
        Post.post_objects.create(
            title="!@#$%^&*()",
            content="This is the content of the post with an invalid title.",
            author=user,
        )
    assert str(exc_info.value) == "Invalid title. Unable to generate a valid slug."


@pytest.mark.django_db
def test_post_markdown_rendering(user, settings):
    with pytest.raises(KeyError):
        assert settings.DJPRESS_SETTINGS["MARKDOWN_EXTENSIONS"] == []

    # Test case 1: Render markdown with basic formatting
    post1 = Post.post_objects.create(
        title="Post with Markdown",
        content="# Heading\n\nThis is a paragraph with **bold** and *italic* text.",
        author=user,
    )
    expected_html = "<h1>Heading</h1>\n<p>This is a paragraph with <strong>bold</strong> and <em>italic</em> text.</p>"
    assert post1.content_markdown == expected_html


@pytest.mark.django_db
def test_post_truncated_content_markdown(user, settings):
    # Confirm the truncate tag is set according to settings_testing.py
    truncate_tag = "<!--test-more-->"
    assert settings.DJPRESS_SETTINGS["TRUNCATE_TAG"] == truncate_tag

    # Test case 1: Content with "read more" tag
    post1 = Post.post_objects.create(
        title="Post with Read More",
        content=f"This is the intro.\n\n{truncate_tag}\n\nThis is the rest of the content.",
        author=user,
    )
    expected_truncated_content = "<p>This is the intro.</p>"
    assert post1.truncated_content_markdown == expected_truncated_content

    # Test case 2: Content without "read more" tag
    post2 = Post.post_objects.create(
        title="Post without Read More",
        content="This is the entire content.",
        author=user,
    )
    expected_truncated_content = "<p>This is the entire content.</p>"
    assert post2.truncated_content_markdown == expected_truncated_content


@pytest.mark.django_db
def test_post_is_truncated_property(user, settings):
    # Confirm the truncate tag is set according to settings_testing.py
    truncate_tag = "<!--test-more-->"
    assert settings.DJPRESS_SETTINGS["TRUNCATE_TAG"] == truncate_tag

    # Test case 1: Content with truncate tag
    post1 = Post.post_objects.create(
        title="Post with Truncate Tag",
        content=f"This is the intro.{truncate_tag}This is the rest of the content.",
        author=user,
    )
    assert post1.is_truncated is True

    # Test case 2: Content without truncate tag
    post2 = Post.post_objects.create(
        title="Post without Truncate Tag",
        content="This is the entire content.",
        author=user,
    )
    assert post2.is_truncated is False

    # Test case 3: Content with truncate tag at the beginning
    post3 = Post.post_objects.create(
        title="Post with Truncate Tag at the Beginning",
        content=f"{truncate_tag}This is the content.",
        author=user,
    )
    assert post3.is_truncated is True

    # Test case 4: Content with truncate tag at the end
    post4 = Post.post_objects.create(
        title="Post with Truncate Tag at the End",
        content=f"This is the content.{truncate_tag}",
        author=user,
    )
    assert post4.is_truncated is True


@pytest.mark.django_db
def test_post_permalink(user, settings):
    post = Post(
        title="Test Post",
        slug="test-post",
        content="This is a test post.",
        author=user,
        date=timezone.make_aware(timezone.datetime(2024, 1, 1)),
        status="published",
        post_type="post",
    )

    # Confirm the post prefix and permalink settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["POST_PREFIX"] == "test-posts"
    assert post.permalink == "test-posts/test-post"

    # Test with no post prefix
    settings.DJPRESS_SETTINGS["POST_PREFIX"] = ""
    assert post.permalink == "test-post"

    # Test with text, year, month, day post prefix
    settings.DJPRESS_SETTINGS["POST_PREFIX"] = "test-posts/{{ year }}/{{ month }}/{{ day }}"
    assert post.permalink == "test-posts/2024/01/01/test-post"

    # Test with text, year, month post prefix
    settings.DJPRESS_SETTINGS["POST_PREFIX"] = "test-posts/{{ year }}/{{ month }}"
    assert post.permalink == "test-posts/2024/01/test-post"

    # Test with text, year post prefix
    settings.DJPRESS_SETTINGS["POST_PREFIX"] = "test-posts/{{ year }}"
    assert post.permalink == "test-posts/2024/test-post"

    # Test with year, month, day post prefix
    settings.DJPRESS_SETTINGS["POST_PREFIX"] = "{{ year }}/{{ month }}/{{ day }}"
    assert post.permalink == "2024/01/01/test-post"

    # Test with year, month post prefix
    settings.DJPRESS_SETTINGS["POST_PREFIX"] = "{{ year }}/{{ month }}"
    assert post.permalink == "2024/01/test-post"

    # Test with year post prefix
    settings.DJPRESS_SETTINGS["POST_PREFIX"] = "{{ year }}"
    assert post.permalink == "2024/test-post"


@pytest.mark.django_db
def test_get_published_posts_by_author(user):
    # Create some published posts by the test user
    post1 = Post.post_objects.create(
        title="Post 1",
        content="Content of post 1.",
        author=user,
        status="published",
    )
    post2 = Post.post_objects.create(
        title="Post 2",
        content="Content of post 2.",
        author=user,
        status="published",
    )

    # Create a draft post by the test user
    draft_post = Post.post_objects.create(
        title="Draft Post",
        content="Content of draft post.",
        author=user,
        status="draft",
    )

    # Create a future post by the test user
    future_post = Post.post_objects.create(
        title="Future Post",
        content="Content of future post.",
        author=user,
        status="published",
        date=timezone.now() + timezone.timedelta(days=1),
    )

    # Call the method being tested
    published_posts = Post.post_objects.get_published_posts_by_author(user)

    # Assert that only the published posts by the test user are returned
    assert post1 in published_posts
    assert post2 in published_posts
    assert draft_post not in published_posts
    assert future_post not in published_posts


@pytest.mark.django_db
def test_page_permalink(test_page1):
    assert test_page1.permalink == "test-page1"


@pytest.mark.django_db
def test_page_permalink_parent(test_page1, test_page2):
    test_page1.parent = test_page2
    test_page1.save()
    assert test_page1.permalink == "test-page2/test-page1"


@pytest.mark.django_db
def test_get_recent_published_posts(user, settings):
    """Test that the get_recent_published_posts method returns the correct posts."""
    # Confirm settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    # Create some published posts
    post1 = Post.objects.create(title="Post 1", status="published", author=user, content="Test post")
    post2 = Post.objects.create(title="Post 2", status="published", author=user, content="Test post")
    post3 = Post.objects.create(title="Post 3", status="published", author=user, content="Test post")

    # Call the method being tested
    recent_posts = Post.post_objects.get_recent_published_posts()

    # Assert that the correct posts are returned
    assert list(recent_posts) == [post3, post2, post1]

    # Test case 2: Limit the number of posts returned
    settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] = 2

    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 2

    # Call the method being tested again
    recent_posts = Post.post_objects.get_recent_published_posts()

    # Assert that the correct posts are returned
    assert list(recent_posts) == [post3, post2]
    assert post1 not in recent_posts


@pytest.mark.django_db
def test_get_published_page_by_slug(test_page1):
    """Test that the get_published_page_by_slug method returns the correct page."""
    assert test_page1 == Post.page_objects.get_published_page_by_slug("test-page1")

    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_slug("non-existent-page")


@pytest.mark.django_db
def test_get_published_pages(test_page1, test_page2, test_page3, test_page4, test_page5):
    """Test that the get_published_pages method returns the correct pages."""
    assert list(Post.page_objects.get_published_pages()) == [test_page1, test_page2, test_page3, test_page4, test_page5]

    test_page1.status = "draft"
    test_page1.save()
    assert list(Post.page_objects.get_published_pages()) == [test_page2, test_page3, test_page4, test_page5]

    test_page2.parent = test_page1
    test_page2.save()
    assert list(Post.page_objects.get_published_pages()) == [test_page3, test_page4, test_page5]

    test_page3.date = timezone.now() + timezone.timedelta(days=1)
    test_page3.save()
    assert list(Post.page_objects.get_published_pages()) == [test_page4, test_page5]

    test_page4.parent = test_page3
    test_page4.save()
    test_page5.parent = test_page4
    test_page5.save()
    assert list(Post.page_objects.get_published_pages()) == []


@pytest.mark.django_db
def test_get_published_page_by_path_top_level(test_page1):
    """Test that the get_published_page_by_path method returns the correct page."""

    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page1")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"test-page1/")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page1/")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"//////test-page1/////")


@pytest.mark.django_db
def test_get_published_page_by_path_parent(test_page1, test_page2):
    """Test that the get_published_page_by_path method returns the correct page."""
    test_page1.parent = test_page2
    test_page1.save()

    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page2/test-page1")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"test-page2/test-page1/")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page2/test-page1/")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"//////test-page2/test-page1/////")


@pytest.mark.django_db
def test_get_published_page_by_path_grandparent(test_page1, test_page2, test_page3):
    """Test that the get_published_page_by_path method returns the correct page."""
    test_page1.parent = test_page2
    test_page1.save()
    test_page2.parent = test_page3
    test_page2.save()

    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page3/test-page2/test-page1")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"test-page3/test-page2/test-page1/")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page3/test-page2/test-page1/")
    assert test_page1 == Post.page_objects.get_published_page_by_path(f"//////test-page3/test-page2/test-page1/////")


@pytest.mark.django_db
def test_get_published_page_with_draft_parent(test_page1, test_page2, test_page3):
    """Test that the get_published_page_by_path method returns the correct page."""
    test_page1.parent = test_page2
    test_page1.save()

    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page2/test-page1")

    test_page2.status = "draft"
    test_page2.save()

    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path(f"/test-page2/test-page1")


@pytest.mark.django_db
def test_get_published_page_with_draft_grandparent(test_page1, test_page2, test_page3):
    """Test that the get_published_page_by_path method returns the correct page."""
    test_page1.parent = test_page2
    test_page1.save()
    test_page2.parent = test_page3
    test_page2.save()

    assert test_page1 == Post.page_objects.get_published_page_by_path(f"/test-page3/test-page2/test-page1")

    test_page3.status = "draft"
    test_page3.save()

    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path(f"/test-page3/test-page2/test-page1")


@pytest.mark.django_db
def test_get_non_existent_page_by_path():
    """Test that the get_published_page_by_path method raises a PageNotFoundError."""
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("non-existent-page")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("non-existint-parent/non-existent-page")


@pytest.mark.django_db
def test_get_non_existent_page_by_path_with_parent(test_page1):
    """Test that the get_published_page_by_path method raises a PageNotFoundError."""
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page1/non-existent-page")


@pytest.mark.django_db
def test_get_valid_page_with_wrong_parent(test_page1, test_page2, test_page3):
    """Test that the get_published_page_by_path method raises a PageNotFoundError."""
    test_page1.parent = test_page2
    test_page1.save()

    assert test_page1 == Post.page_objects.get_published_page_by_path("test-page2/test-page1")
    assert test_page2 == Post.page_objects.get_published_page_by_path("test-page2")
    assert test_page3 == Post.page_objects.get_published_page_by_path("test-page3")

    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page3/test-page1")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page3/test-page2")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page3/test-page3")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page1/test-page1")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page1/test-page2")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page1/test-page3")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page2/test-page2")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page2/test-page3")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page2/test-page1/test-page3")


@pytest.mark.django_db
def test_get_valid_page_with_wrong_grandparent(test_page1, test_page2, test_page3):
    """Test that the get_published_page_by_path method raises a PageNotFoundError."""
    test_page1.parent = test_page2
    test_page1.save()
    test_page2.parent = test_page3
    test_page2.save()

    assert test_page1 == Post.page_objects.get_published_page_by_path("test-page3/test-page2/test-page1")
    assert test_page2 == Post.page_objects.get_published_page_by_path("test-page3/test-page2")
    assert test_page3 == Post.page_objects.get_published_page_by_path("test-page3")

    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page3/test-page1/test-page2")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page3/test-page2/test-page2")
    with pytest.raises(PageNotFoundError):
        Post.page_objects.get_published_page_by_path("test-page1/test-page2/test-page3")


@pytest.mark.django_db
def test_get_cached_published_posts(settings, monkeypatch, test_post1, test_post2):
    """Test that the get_published_pages method returns the correct pages."""
    # Confirm settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    # Turn on caching and check if the setting is correct
    settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] = True
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is True

    # Mock cache.set and cache.get
    mock_cache_set = Mock()
    mock_cache_get = Mock(return_value=None)
    monkeypatch.setattr(cache, "set", mock_cache_set)
    monkeypatch.setattr(cache, "get", mock_cache_get)

    # First call should set the cache
    assert list(Post.post_objects.get_recent_published_posts()) == [test_post2, test_post1]
    assert mock_cache_set.called

    # Get the arguments passed to cache.set
    args, kwargs = mock_cache_set.call_args

    # Mock cache.get to return the cached value
    mock_cache_get.return_value = [test_post2, test_post1]

    # Second call should read from the cache
    assert list(Post.post_objects.get_recent_published_posts()) == [test_post2, test_post1]
    assert mock_cache_get.called

    # Verify that cache.get was called with the expected key
    cache_key = args[0]  # Assuming the cache key is the first argument to cache.set
    mock_cache_get.assert_called_with(cache_key)


@pytest.mark.django_db
def test_get_cached_future_published_posts(user, settings, mock_timezone_now, monkeypatch, test_post1):
    """Test that the def _get_cached_recent_published_posts method sets the correct timeout.

    This is a complicated test that involves mocking the timezone.now function and the cache.set function.

    The mocking can tell what arguments were passed to cache.set and if the timeout is set correctly.
    """
    # Confirm settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    # Turn on caching and check if the setting is correct
    settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] = True
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is True

    assert mock_timezone_now == timezone.now()

    post_date = mock_timezone_now + timezone.timedelta(hours=2)

    print(f"{mock_timezone_now=}")
    print(f"{post_date=}")

    Post.post_objects.create(
        title="Test Post",
        slug="test-post",
        content="This is a test post.",
        author=user,
        date=post_date,
        status="published",
        post_type="post",
    )

    mock_cache_set = Mock()
    monkeypatch.setattr(cache, "set", mock_cache_set)

    queryset = cache.get(PUBLISHED_POSTS_CACHE_KEY)
    assert queryset is None

    queryset = Post.post_objects.get_recent_published_posts()
    assert len(queryset) == 0

    # Check if cache.set was called
    assert mock_cache_set.called

    # Get the arguments passed to cache.set
    args, kwargs = mock_cache_set.call_args

    # Check if the timeout is correct (should be close to 2 hours)
    expected_timeout = 7200  # 2 hours in seconds

    print(f"{args=}")
    print(f"{kwargs=}")

    actual_timeout = kwargs.get("timeout") or args[2]  # timeout might be a kwarg or the third positional arg
    assert abs(actual_timeout - expected_timeout) < 5  # Allow a small margin of error


@pytest.fixture
def mock_cache(monkeypatch):
    mock_cache_get = Mock()
    mock_cache_set = Mock()
    monkeypatch.setattr(cache, "get", mock_cache_get)
    monkeypatch.setattr(cache, "set", mock_cache_set)
    return mock_cache_get, mock_cache_set


@pytest.mark.django_db
def test_get_recent_published_posts_cache_miss(mock_cache, settings, test_post1, test_post2, test_post3):
    """First time calling the get_recent_published_posts method should result in a cache miss."""
    mock_cache_get, mock_cache_set = mock_cache

    # Simulate cache miss
    mock_cache_get.return_value = None

    # Enable caching
    settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] = True

    # Call the method
    queryset = Post.post_objects.get_recent_published_posts()

    # Verify cache.set is called
    assert mock_cache_set.called
    args, kwargs = mock_cache_set.call_args
    cache_key = args[0]
    cached_queryset = args[1]
    timeout = kwargs["timeout"]

    # Verify the queryset is correct
    assert list(queryset) == [test_post3, test_post2, test_post1]
    assert list(cached_queryset) == [test_post3, test_post2, test_post1]


@pytest.mark.django_db
def test_get_recent_published_posts_cache_hit(mock_cache, settings, test_post1, test_post2, test_post3):
    """Test that the get_recent_published_posts method returns the correct posts from the cache."""
    # Confirm settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    mock_cache_get, mock_cache_set = mock_cache

    # Simulate cache hit
    mock_cache_get.return_value = [test_post3, test_post2, test_post1]

    # Enable caching
    settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] = True

    # Call the method
    cached_queryset = Post.post_objects.get_recent_published_posts()

    # Verify cache.get is called
    mock_cache_get.assert_called_with(PUBLISHED_POSTS_CACHE_KEY)
    assert list(cached_queryset) == [test_post3, test_post2, test_post1]

    new_queryset = Post.post_objects.get_recent_published_posts()

    # Verify cache.set is not called again
    assert not mock_cache_set.called


@pytest.mark.django_db
def test_get_recent_published_posts_cache_hit_2_posts(mock_cache, settings, test_post1, test_post2):
    """Test that the get_recent_published_posts method returns the correct posts from the cache."""
    # Confirm settings are set according to settings_testing.py
    assert settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] is False
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    mock_cache_get, mock_cache_set = mock_cache

    # Simulate cache hit
    mock_cache_get.return_value = [test_post2, test_post1]

    # Enable caching
    settings.DJPRESS_SETTINGS["CACHE_RECENT_PUBLISHED_POSTS"] = True
    settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] = 2

    # Call the method
    cached_queryset = Post.post_objects.get_recent_published_posts()

    # Verify cache.get is called
    mock_cache_get.assert_called_with(PUBLISHED_POSTS_CACHE_KEY)
    assert list(cached_queryset) == [test_post2, test_post1]

    new_queryset = Post.post_objects.get_recent_published_posts()

    # Verify cache.set is not called again
    assert not mock_cache_set.called


@pytest.mark.django_db
def test_get_cached_recent_published_posts_cache_miss(mock_cache, test_post1, test_post2):
    """Test that the _get_cached_recent_published_posts method sets the correct cache key and value."""
    mock_cache_get, mock_cache_set = mock_cache

    # Simulate cache miss
    mock_cache_get.return_value = None

    # Call the method
    queryset = Post.post_objects._get_cached_recent_published_posts()

    # Verify cache.set is called
    assert mock_cache_set.called
    args, kwargs = mock_cache_set.call_args
    cache_key = args[0]
    cached_queryset = args[1]
    timeout = kwargs["timeout"]

    # Verify the queryset is correct
    assert list(queryset) == [test_post2, test_post1]
    assert list(cached_queryset) == [test_post2, test_post1]


@pytest.mark.django_db
def test_get_cached_recent_published_posts_cache_hit(mock_cache, settings, test_post1, test_post2, test_post3):
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3

    mock_cache_get, mock_cache_set = mock_cache

    # Simulate cache hit
    mock_cache_get.return_value = [test_post3, test_post2, test_post1]

    # Call the method
    cached_queryset = Post.post_objects._get_cached_recent_published_posts()

    # Verify cache.get is called
    mock_cache_get.assert_called_with(PUBLISHED_POSTS_CACHE_KEY)
    assert list(cached_queryset) == [test_post3, test_post2, test_post1]

    # Verify cache.set is not called again
    assert not mock_cache_set.called


@pytest.mark.django_db
def test_get_cached_recent_published_posts_cache_hit_2_posts(mock_cache, settings, test_post1, test_post2):
    assert settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] == 3
    # Change the number of posts to 2
    settings.DJPRESS_SETTINGS["RECENT_PUBLISHED_POSTS_COUNT"] = 2

    mock_cache_get, mock_cache_set = mock_cache

    # Simulate cache hit
    mock_cache_get.return_value = [test_post2, test_post1]

    # Call the method
    cached_queryset = Post.post_objects._get_cached_recent_published_posts()

    # Verify cache.get is called
    mock_cache_get.assert_called_with(PUBLISHED_POSTS_CACHE_KEY)
    assert list(cached_queryset) == [test_post2, test_post1]

    # Verify cache.set is not called again
    assert not mock_cache_set.called


@pytest.mark.django_db
def test_post_clean_valid_parent(test_page1, test_page2):
    test_page1.parent = test_page2
    test_page1.clean()
    assert test_page1.parent == test_page2


@pytest.mark.django_db
def test_post_clean_self_parent(test_page1):
    test_page1.parent = test_page1
    with pytest.raises(ValidationError) as exc_info:
        test_page1.clean()
    assert "Circular reference detected in page hierarchy." in str(exc_info.value)


@pytest.mark.django_db
def test_post_clean_circular_reference(test_page1, test_page2):
    test_page1.parent = test_page2
    test_page1.clean()
    assert test_page1.parent == test_page2

    # Create a circular reference
    test_page2.parent = test_page1
    with pytest.raises(ValidationError) as exc_info:
        test_page2.clean()
    assert "Circular reference detected in page hierarchy." in str(exc_info.value)


@pytest.mark.django_db
def test_post_clean_circular_reference_extra_level(test_page1, test_page2, test_page3):
    test_page1.parent = test_page2
    test_page1.clean()
    assert test_page1.parent == test_page2

    test_page2.parent = test_page3
    test_page2.clean()
    assert test_page2.parent == test_page3

    # Create a circular reference
    test_page3.parent = test_page1
    with pytest.raises(ValidationError) as exc_info:
        test_page3.clean()
    assert "Circular reference detected in page hierarchy." in str(exc_info.value)


@pytest.mark.django_db
def test_full_page_path_no_parent(test_page1):
    assert test_page1.full_page_path == "test-page1"


@pytest.mark.django_db
def test_get_full_page_path_with_parent(test_page1, test_page2):
    test_page1.parent = test_page2
    assert test_page1.full_page_path == "test-page2/test-page1"


@pytest.mark.django_db
def test_get_full_page_path_with_grandparent(test_page1, test_page2, test_page3):
    test_page1.parent = test_page2
    test_page2.parent = test_page3
    assert test_page1.full_page_path == "test-page3/test-page2/test-page1"


@pytest.mark.django_db
def test_page_get_page_tree_no_children(test_page1, test_page2, test_page3, test_page4):
    expected_tree = [
        {"page": test_page1, "children": []},
        {"page": test_page2, "children": []},
        {"page": test_page3, "children": []},
        {"page": test_page4, "children": []},
    ]
    assert list(Post.page_objects.get_page_tree()) == expected_tree


@pytest.mark.django_db
def test_page_get_page_tree_with_children(test_page1, test_page2, test_page3, test_page4):
    test_page1.parent = test_page2
    test_page1.save()
    test_page3.parent = test_page2
    test_page3.save()

    expected_tree = [
        {
            "page": test_page2,
            "children": [
                {"page": test_page1, "children": []},
                {"page": test_page3, "children": []},
            ],
        },
        {"page": test_page4, "children": []},
    ]
    assert Post.page_objects.get_page_tree() == expected_tree


@pytest.mark.django_db
def test_page_get_page_tree_with_grandchildren(test_page1, test_page2, test_page3, test_page4, test_page5):
    test_page1.parent = test_page2
    test_page1.save()
    test_page3.parent = test_page2
    test_page3.save()
    test_page2.parent = test_page5
    test_page2.save()

    expected_tree = [
        {"page": test_page4, "children": []},
        {
            "page": test_page5,
            "children": [
                {
                    "page": test_page2,
                    "children": [
                        {"page": test_page1, "children": []},
                        {"page": test_page3, "children": []},
                    ],
                },
            ],
        },
    ]
    assert Post.page_objects.get_page_tree() == expected_tree


@pytest.mark.django_db
def test_page_get_page_tree_with_grandchildren_parent_with_future_date(
    test_page1, test_page2, test_page3, test_page4, test_page5
):
    """Test complex page structure.

    test_page5
    ├── test_page2 (future) = should be unpublished
    │   ├── test_page1 = should be unpublished
    │   └── test_page3 = should be unpublished
    test_page4
    """
    test_page1.parent = test_page2
    test_page1.save()
    test_page3.parent = test_page2
    test_page3.save()
    test_page2.parent = test_page5
    test_page2.date = timezone.now() + timezone.timedelta(days=1)
    test_page2.save()

    assert test_page2.is_published is False

    expected_tree = [
        {"page": test_page4, "children": []},
        {"page": test_page5, "children": []},
    ]
    assert Post.page_objects.get_page_tree() == expected_tree


@pytest.mark.django_db
def test_page_get_page_tree_with_grandchildren_parent_with_status_draft(
    test_page1, test_page2, test_page3, test_page4, test_page5
):
    """Test complex page structure.

    test_page5
    ├── test_page2 (future) = should be unpublished
    │   ├── test_page1 = should be unpublished
    │   └── test_page3 = should be unpublished
    test_page4
    """
    test_page1.parent = test_page2
    test_page1.save()
    test_page3.parent = test_page2
    test_page3.save()
    test_page2.parent = test_page5
    test_page2.status = "draft"
    test_page2.save()

    expected_tree = [
        {"page": test_page4, "children": []},
        {"page": test_page5, "children": []},
    ]
    assert Post.page_objects.get_page_tree() == expected_tree


@pytest.mark.django_db
def test_page_order_menu_order(test_page1, test_page2, test_page3, test_page4, test_page5):
    test_page1.menu_order = 1
    test_page1.save()
    test_page2.menu_order = 2
    test_page2.save()
    test_page3.menu_order = 3
    test_page3.save()
    test_page4.menu_order = 4
    test_page4.save()
    test_page5.menu_order = 5
    test_page5.save()

    expected_order = [test_page1, test_page2, test_page3, test_page4, test_page5]

    assert list(Post.page_objects.get_published_pages()) == expected_order


@pytest.mark.django_db
def test_page_order_title(test_page1, test_page2, test_page3, test_page4, test_page5):
    test_page1.menu_order = 1
    test_page1.save()
    test_page2.menu_order = 1
    test_page2.save()
    test_page3.menu_order = 1
    test_page3.save()
    test_page4.menu_order = 1
    test_page4.save()
    test_page5.menu_order = 1
    test_page5.save()

    expected_order = [test_page1, test_page2, test_page3, test_page4, test_page5]

    assert list(Post.page_objects.get_published_pages()) == expected_order


@pytest.mark.django_db
def test_page_is_published_parent_published_page_draft(test_page1, test_page2):
    test_page1.parent = test_page2
    test_page1.save()
    assert test_page1.is_published is True
    assert test_page2.is_published is True

    test_page1.status = "draft"
    test_page1.save()
    assert test_page1.is_published is False
    assert test_page2.is_published is True


@pytest.mark.django_db
def test_page_is_published(test_page1, test_page2, test_page3, test_page4, test_page5):
    # All pages are published
    assert test_page1.is_published is True
    assert test_page2.is_published is True
    assert test_page3.is_published is True
    assert test_page4.is_published is True
    assert test_page5.is_published is True

    # Change test_page1 to be draft - test_page1 will be unpublished
    test_page1.status = "draft"
    test_page1.save()
    assert test_page1.is_published is False

    # Change test_page2 to have test_page1 as the parent - now both will be unpublished
    test_page2.parent = test_page1
    test_page2.save()
    assert test_page1.is_published is False
    assert test_page2.is_published is False

    # Change test_page1 to be published again - test_page1 and the child test_page2 will be published again
    test_page1.status = "published"
    test_page1.save()
    assert test_page1.is_published is True
    assert test_page2.is_published is True

    # Now change test_page2 to draft - test_page1 will still be published, but test_page2 will be unpublished
    test_page2.status = "draft"
    test_page2.save()
    assert test_page1.is_published is True
    assert test_page2.is_published is False

    # Change test_page2 to published - now both will be published again
    test_page2.status = "published"
    test_page2.save()
    assert test_page2.is_published is True
    assert test_page1.is_published is True

    # Change test_page3 to be in the future - test_page3 and the child test_page2 will be unpublished
    test_page3.date = timezone.now() + timezone.timedelta(days=1)
    test_page3.save()
    assert test_page3.is_published is False
    assert test_page2.is_published is True
    assert test_page1.is_published is True

    # Change test_page2 to have test_page3 as the parent - test_page3 and test_page2 will be unpublished and test_page1 will be published
    test_page2.parent = test_page3
    test_page2.save()
    assert test_page3.is_published is False
    assert test_page2.is_published is False
    assert test_page1.is_published is True

    # Change test_page3 to be published again - test_page3 and the child test_page2 will be published
    test_page3.date = timezone.now()
    test_page3.save()
    assert test_page2.is_published is True
    assert test_page3.is_published is True

    # Change test_page3 to have test_page4 as the parent - test_page4, test_page3 and test_page2 will be published
    test_page3.parent = test_page4
    test_page3.save()
    assert test_page3.is_published is True
    assert test_page2.is_published is True
    assert test_page4.is_published is True

    # Change test_page 4 to have test_page5 as the parent - test_page5, test_page4, test_page3 and test_page2 will be published
    test_page4.parent = test_page5
    test_page4.save()
    assert test_page2.is_published is True
    assert test_page3.is_published is True
    assert test_page4.is_published is True
    assert test_page5.is_published is True

    # Change test_page5 to be draft - test_page5, test_page4, test_page3 and test_page2 will be unpublished
    test_page5.status = "draft"
    test_page5.save()
    assert test_page5.is_published is False
    assert test_page3.is_published is False
    assert test_page4.is_published is False
    assert test_page5.is_published is False


@pytest.mark.django_db
def test_page_is_parent(test_page1, test_page2):
    assert test_page1.is_parent is False
    assert test_page2.is_parent is False

    test_page1.parent = test_page2
    test_page1.save()
    assert test_page2.is_parent is True


@pytest.mark.django_db
def test_page_is_child(test_page1, test_page2):
    assert test_page1.is_child is False
    assert test_page2.is_child is False

    test_page1.parent = test_page2
    test_page1.save()
    assert test_page1.is_child is True


@pytest.mark.django_db
def test_parent_page_cant_have_post_child(test_page1, test_post1):
    test_post1.parent = test_page1
    test_post1.save()

    assert test_post1 not in test_page1.children.all()


@pytest.mark.django_db
def test_post_parent_is_none(test_post1, test_page1):
    test_post1.parent = test_page1
    test_post1.save()

    assert test_post1.parent is None


@pytest.mark.django_db
def test_post_get_years(test_post1, test_post2, test_post3):
    # type should be a queryset
    assert isinstance(Post.post_objects.get_years(), QuerySet)
    # Queryset should have 1 item
    assert len(Post.post_objects.get_years()) == 1
    # The item should be the year of the post
    assert Post.post_objects.get_years()[0].year == test_post1.date.year

    test_post2.date = timezone.make_aware(timezone.datetime(2023, 1, 1, 12, 0, 0))
    test_post2.save()

    # Queryset should have 2 items
    assert len(Post.post_objects.get_years()) == 2
    # The items should be the years of the posts
    assert Post.post_objects.get_years()[0].year == test_post2.date.year
    assert Post.post_objects.get_years()[1].year == test_post1.date.year

    test_post3.date = timezone.make_aware(timezone.datetime(2022, 1, 1, 12, 0, 0))
    test_post3.save()

    # Queryset should have 3 items
    assert len(Post.post_objects.get_years()) == 3
    # The items should be the years of the posts
    assert Post.post_objects.get_years()[0].year == test_post3.date.year
    assert Post.post_objects.get_years()[1].year == test_post2.date.year
    assert Post.post_objects.get_years()[2].year == test_post1.date.year

    # Change a post to draft status
    test_post1.status = "draft"
    test_post1.save()

    # Queryset should have 2 items
    assert len(Post.post_objects.get_years()) == 2
    # The items should be the years of the posts
    assert Post.post_objects.get_years()[0].year == test_post3.date.year
    assert Post.post_objects.get_years()[1].year == test_post2.date.year


@pytest.mark.django_db
def test_post_get_months(test_post1, test_post2, test_post3):
    months = Post.post_objects.get_months(test_post1.date.year)

    # type should be a queryset
    assert isinstance(months, QuerySet)
    # Queryset should have 1 item - all three posts are in the same year and month
    assert len(months) == 1
    # The item should be the month of the post
    assert months[0].month == test_post1.date.month

    # Set specific dates for each of the posts
    test_post1.date = timezone.make_aware(timezone.datetime(2022, 1, 1, 12, 0, 0))
    test_post1.save()
    test_post2.date = timezone.make_aware(timezone.datetime(2022, 2, 1, 12, 0, 0))
    test_post2.save()
    test_post3.date = timezone.make_aware(timezone.datetime(2022, 3, 1, 12, 0, 0))
    test_post3.save()

    months = Post.post_objects.get_months(test_post1.date.year)

    # Queryset should have 3 items
    assert len(months) == 3
    # The items should be the months of the posts
    assert months[0].month == test_post1.date.month
    assert months[1].month == test_post2.date.month
    assert months[2].month == test_post3.date.month

    # Change a post to draft status
    test_post1.status = "draft"
    test_post1.save()

    months = Post.post_objects.get_months(test_post1.date.year)

    # Queryset should have 2 items
    assert len(months) == 2
    # The items should be the months of the posts
    assert months[0].month == test_post2.date.month
    assert months[1].month == test_post3.date.month


@pytest.mark.django_db
def test_post_get_days(test_post1, test_post2, test_post3):
    days = Post.post_objects.get_days(test_post1.date.year, test_post1.date.month)

    # type should be a queryset
    assert isinstance(days, QuerySet)
    # Queryset should have 1 item - all three posts are in the same year and month
    assert len(days) == 1

    # Set specific dates for each of the posts
    test_post1.date = timezone.make_aware(timezone.datetime(2022, 1, 1, 12, 0, 0))
    test_post1.save()
    test_post2.date = timezone.make_aware(timezone.datetime(2022, 1, 2, 12, 0, 0))
    test_post2.save()
    test_post3.date = timezone.make_aware(timezone.datetime(2022, 1, 3, 12, 0, 0))
    test_post3.save()

    days = Post.post_objects.get_days(test_post1.date.year, test_post1.date.month)

    # Queryset should have 3 items
    assert len(days) == 3
    # The items should be the days of the posts
    assert days[0].day == test_post1.date.day
    assert days[1].day == test_post2.date.day
    assert days[2].day == test_post3.date.day

    # Change a post to draft status
    test_post1.status = "draft"
    test_post1.save()

    days = Post.post_objects.get_days(test_post1.date.year, test_post1.date.month)

    # Queryset should have 2 items
    assert len(days) == 2
    # The items should be the days of the posts
    assert days[0].day == test_post2.date.day
    assert days[1].day == test_post3.date.day


@pytest.mark.django_db
def test_get_year_last_modified(test_post1, test_post2, test_post3):
    # Should match the modified date of the last post in the list - i.e. most recent post
    assert Post.post_objects.get_year_last_modified(test_post1.date.year) == test_post3.modified_date

    # Change test_post3 to draft and it should now match test_post2
    test_post3.status = "draft"
    test_post3.save()
    assert Post.post_objects.get_year_last_modified(test_post1.date.year) == test_post2.modified_date

    # Changetest_post2 to future date and it should now match test_post1
    test_post2.date = timezone.now() + timezone.timedelta(days=1)
    test_post2.save()
    assert Post.post_objects.get_year_last_modified(test_post1.date.year) == test_post1.modified_date


@pytest.mark.django_db
def test_get_month_last_modified(test_post1, test_post2, test_post3):
    # Should match the modified date of the last post in the list - i.e. most recent post
    assert (
        Post.post_objects.get_month_last_modified(test_post1.date.year, test_post1.date.month)
        == test_post3.modified_date
    )

    # Change test_post3 to draft and it should now match test_post2
    test_post3.status = "draft"
    test_post3.save()
    assert (
        Post.post_objects.get_month_last_modified(test_post1.date.year, test_post1.date.month)
        == test_post2.modified_date
    )

    # Changetest_post2 to future date and it should now match test_post1
    test_post2.date = timezone.now() + timezone.timedelta(days=1)
    test_post2.save()
    assert (
        Post.post_objects.get_month_last_modified(test_post1.date.year, test_post1.date.month)
        == test_post1.modified_date
    )


@pytest.mark.django_db
def test_get_day_last_modified(test_post1, test_post2, test_post3):
    # Should match the modified date of the last post in the list - i.e. most recent post
    assert (
        Post.post_objects.get_day_last_modified(test_post1.date.year, test_post1.date.month, test_post1.date.day)
        == test_post3.modified_date
    )

    # Change test_post3 to draft and it should now match test_post2
    test_post3.status = "draft"
    test_post3.save()
    assert (
        Post.post_objects.get_day_last_modified(test_post1.date.year, test_post1.date.month, test_post1.date.day)
        == test_post2.modified_date
    )

    # Changetest_post2 to future date and it should now match test_post1
    test_post2.date = timezone.now() + timezone.timedelta(days=1)
    test_post2.save()
    assert (
        Post.post_objects.get_day_last_modified(test_post1.date.year, test_post1.date.month, test_post1.date.day)
        == test_post1.modified_date
    )



"""Tests for the PostAdmin class."""

import pytest
from django.contrib.admin import site
from django.utils import timezone
from datetime import timedelta

from djpress.admin import PostAdmin
from djpress.models import Post


@pytest.mark.django_db
def test_published_status(test_post1):
    """Test the published_status method returns correct boolean values."""
    # Get the admin class
    post_admin = PostAdmin(Post, site)

    # Test published post
    assert post_admin.published_status(test_post1) is True

    # Chnage to draft
    test_post1.status = "draft"
    test_post1.save()
    assert post_admin.published_status(test_post1) is False

    # Change to future post
    test_post1.status = "published"
    test_post1.date = timezone.now() + timedelta(days=1)
    test_post1.save()
    assert post_admin.published_status(test_post1) is False


@pytest.mark.django_db
def test_formatted_date(test_post1):
    """Test the formatted_date method returns the correct date."""
    # Get the admin class
    post_admin = PostAdmin(Post, site)

    # Test formatted date
    assert post_admin.formatted_date(test_post1) == test_post1.date.strftime("%Y-%m-%d %H:%M")

    # Test future post
    test_post1.date = timezone.now() + timedelta(days=1)
    test_post1.save()

    assert (
        post_admin.formatted_date(test_post1)
        == f'<span style="color: #666;">{test_post1.date.strftime("%Y-%m-%d %H:%M")} (Scheduled)</span>'
    )



"""URL configuration for config project."""

from django.contrib import admin
from django.contrib.sitemaps.views import sitemap
from django.urls import include, path

from djpress.sitemaps import (
    CategorySitemap,
    DateBasedSitemap,
    PageSitemap,
    PostSitemap,
)

# Define your sitemaps dictionary
sitemaps = {
    "posts": PostSitemap,
    "pages": PageSitemap,
    "categories": CategorySitemap,
    "archives": DateBasedSitemap,
}

urlpatterns = [
    path("admin/", admin.site.urls),
    path("__debug__/", include("debug_toolbar.urls")),
    path("sitemap.xml", sitemap, {"sitemaps": sitemaps}, name="django.contrib.sitemaps.views.sitemap"),
    path("", include("djpress.urls")),
]



"""Plugin system for DJ Press."""

import contextlib  # Ruff: SIM105
from enum import Enum
from typing import Any

from django.utils.module_loading import import_string

from djpress.conf import settings as djpress_settings


# Hook definitions
class Hooks(Enum):
    """Available hook points in DJ Press."""

    PRE_RENDER_CONTENT = "pre_render_content"
    POST_RENDER_CONTENT = "post_render_content"
    POST_SAVE_POST = "post_save_post"


class PluginRegistry:
    """A registry for plugins.

    Handles plugin discovery, loading, and hook management.
    """

    def __init__(self) -> None:
        """Initialize the registry."""
        self.plugins = []
        self.hooks = {}
        self._loaded = False

    def register_hook(self, hook_name: Hooks | str, callback: callable) -> None:
        """Register a callback function for a specific hook.

        Args:
            hook_name (Hooks | str): The name of the hook - this can be referenced either as the plain text name of
                the hook (e.g. `"pre_render_content"`) or as a Hooks enum member (e.g. `Hooks.PRE_RENDER_CONTENT`).
            callback (callable): The function to call when the hook is triggered.

        Raises:
            TypeError: If hook_name is not a Hooks enum member.
        """
        # Convert string to Enum if needed, and supress error if not possible
        if isinstance(hook_name, str):
            with contextlib.suppress(ValueError):
                hook_name = Hooks(hook_name)

        if hook_name not in self.hooks:
            self.hooks[hook_name] = []
        self.hooks[hook_name].append(callback)

    def run_hook(self, hook_name: Hooks, value: Any = None, *args: Any, **kwargs: Any) -> Any:  # noqa: ANN401
        """Run all registered callbacks for a given hook.

        Args:
            hook_name (Hooks): The name of the hook to run, this should be in the form of a Hooks enum member.
            value: The value to be modified by the callbacks.
            *args: Additional positional arguments to pass to the callbacks.
            **kwargs: Additional keyword arguments to pass to the callbacks.

        Returns:
            The value after all callbacks have been run.

        Raises:
            TypeError: If hook_name is not a Hooks enum member.
        """
        if not isinstance(hook_name, Hooks):
            msg = f"hook_name must be a Hooks enum member, got {type(hook_name)}"
            raise TypeError(msg)

        if not self._loaded:
            self.load_plugins()

        if hook_name in self.hooks:
            for callback in self.hooks[hook_name]:
                value = callback(value, *args, **kwargs)

        return value

    def load_plugins(self) -> None:
        """Load all plugins.

        Plugins need to be added to the DJPRESS_SETTINGS['PLUGINS'] list.

        Plugins can be added to DJPRESS_SETTINGS['PLUGINS'] in two ways:
        1. Just the package name (e.g., "djpress_example_plugin") which will look for Plugin class in plugin.py
        2. Full path to the plugin class (e.g., "djpress_example_plugin.custom.MyPlugin")
        """
        if self._loaded:
            return

        plugin_names: list = djpress_settings.PLUGINS
        plugin_settings: dict = djpress_settings.PLUGIN_SETTINGS

        for plugin_path in plugin_names:
            plugin_class = self._import_plugin_class(plugin_path)
            plugin = self._instantiate_plugin(plugin_class, plugin_path, plugin_settings)
            self.plugins.append(plugin)

        self._loaded = True

    def _import_plugin_class(self, plugin_path: str) -> type:
        """Import the plugin class from either custom path or standard location.

        Args:
            plugin_path: Either full path to plugin class or just the package name.

        Returns:
            The plugin class.

        Raises:
            ImproperlyConfigured: If plugin cannot be loaded from either location.
        """
        # Try full path first
        try:
            return import_string(plugin_path)
        except ImportError:
            pass

        # Try standard plugin.py location
        try:
            return import_string(f"{plugin_path}.plugin.Plugin")
        except ImportError as exc:
            from django.core.exceptions import ImproperlyConfigured

            msg = (
                f"Could not load plugin '{plugin_path}'. "
                f"Tried both custom path and standard plugin.py location. "
                f"Error: {exc}"
            )
            raise ImproperlyConfigured(msg) from exc

    def _instantiate_plugin(self, plugin_class: type, plugin_path: str, plugin_settings: dict) -> "DJPressPlugin":
        """Create and set up a plugin instance.

        Args:
            plugin_class: The plugin class to instantiate.
            plugin_path: The original plugin path (used for error reporting).
            plugin_settings: Dictionary of settings for all plugins.

        Returns:
            An initialized plugin instance.

        Raises:
            ImproperlyConfigured: If plugin cannot be instantiated or set up.
        """
        try:
            # Get plugin-specific settings
            plugin_name = plugin_class.name if hasattr(plugin_class, "name") else plugin_path
            plugin_config = plugin_settings.get(plugin_name, {})

            # Create plugin instance with optional config
            plugin = plugin_class(config=plugin_config) if plugin_config else plugin_class()

            # Set up the plugin
            plugin.setup(self)

        except Exception as exc:
            from django.core.exceptions import ImproperlyConfigured

            msg = f"Error initializing plugin '{plugin_path}': {exc!s}"
            raise ImproperlyConfigured(msg) from exc

        else:
            return plugin


class DJPressPlugin:
    """Base class for DJ Press plugins."""

    name: str | None = None

    def __init__(self, config: dict | None = None) -> None:
        """Initialize the plugin."""
        if self.name is None:
            msg = "Plugin must define a name"
            raise ValueError(msg)
        self.config = config or {}

    def setup(self, registry: PluginRegistry) -> None:
        """Set up the plugin.

        Args:
            registry (PluginRegistry): The plugin registry.
        """

    def get_data(self) -> dict:
        """Get this plugin's stored data.

        Returns:
            dict: The plugin's stored data, or empty dict if none exists.
        """
        from djpress.models import PluginStorage

        return PluginStorage.objects.get_data(self.name)

    def save_data(self, data: dict) -> None:
        """Save this plugin's data.

        Args:
            data: The data to store for this plugin.
        """
        from djpress.models import PluginStorage

        PluginStorage.objects.save_data(self.name, data)


# Instantiate the global plugin registry
registry = PluginRegistry()



# Generated by Django 5.0.4 on 2024-05-03 11:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="post",
            old_name="content_type",
            new_name="post_type",
        ),
    ]



"""PluginStorage model for storing plugin data in the database."""

from django.db import models


class PluginStorageManager(models.Manager):
    """Manager for the PluginStorage model."""

    def get_data(self, plugin_name: str) -> dict:
        """Get plugin data.

        Retrieve the plugin data from the database. If the plugin data does not exist, return an empty dictionary.

        Args:
            plugin_name (str): The name of the plugin.

        Returns:
            dict: The plugin data.
        """
        try:
            data = self.get(plugin_name=plugin_name)
        except self.model.DoesNotExist:
            return {}
        else:
            return data.plugin_data or {}

    def save_data(self, plugin_name: str, data: dict) -> None:
        """Save plugin data.

        Save or update the plugin data in the database. If no storage exists for this plugin, it will be created.

        Args:
            plugin_name (str): The name of the plugin.
            data (dict): The plugin data.

        Returns:
            None
        """
        storage, created = self.update_or_create(
            plugin_name=plugin_name,
            defaults={"plugin_data": data},
        )


class PluginStorage(models.Model):
    """Model for storing plugin data in the database."""

    plugin_name = models.CharField(max_length=100, unique=True)
    plugin_data = models.JSONField(default=dict)

    # Manager
    objects = PluginStorageManager()

    class Meta:
        """Meta options for the PluginStorage model."""

        verbose_name = "plugin storage"
        verbose_name_plural = "plugin storage"

    def __str__(self) -> str:
        """Return the string representation of the plugin item."""
        return self.plugin_name



"""Custom URL converters."""

from django.conf import settings


class SlugPathConverter:
    """Converter for the DJ Press path.

    The path will only ever contain letters, numbers, underscores, hyphens, and slashes.
    """

    # Regex explained:
    # - [\w/-]+: This matches any word character (alphanumeric or underscore), hyphen, or slash, one or more times.
    @property
    def regex(self) -> str:
        """Return the regex for the path."""
        if settings.APPEND_SLASH:
            return r"^[\w/-]+/$"
        return r"^[\w/-]+$"

    def to_python(self, value: str) -> str:
        """Return the value as a string."""
        return str(value)

    def to_url(self, value: str) -> str:
        """Return the value as a string."""
        return str(value)



import pytest

from django.urls import reverse
from django.contrib.auth.models import User

from djpress.conf import settings
from djpress import url_utils
from djpress.models import Category, Post


@pytest.fixture
def test_post1(user, category1):
    post = Post.objects.create(
        title="Test Post1",
        slug="test-post1",
        content="This is test post 1.",
        author=user,
        status="published",
        post_type="post",
    )
    return post


@pytest.mark.django_db
def test_url_author_enabled(test_post1):
    """Test the author URL."""

    # Confirm settings are set according to settings_testing.py
    assert settings.AUTHOR_ENABLED is True
    assert settings.AUTHOR_PREFIX == "test-url-author"

    author_url = url_utils.get_author_url(test_post1.author)

    assert author_url == f"/{settings.AUTHOR_PREFIX}/testuser/"


@pytest.mark.django_db
def test_url_category_enabled(category1):
    """Test the category URL."""

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_ENABLED is True
    assert settings.CATEGORY_PREFIX == "test-url-category"

    category_url = url_utils.get_category_url(category1)

    assert category_url == f"/{settings.CATEGORY_PREFIX}/test-category1/"



# Generated by Django 5.1.3 on 2024-11-27 23:23

import django.db.models.deletion
import django.db.models.manager
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0007_pluginstorage"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="post",
            options={"default_manager_name": "admin_objects", "verbose_name": "post", "verbose_name_plural": "posts"},
        ),
        migrations.AlterModelManagers(
            name="post",
            managers=[
                ("admin_objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterField(
            model_name="post",
            name="categories",
            field=models.ManyToManyField(blank=True, related_name="_posts", to="djpress.category"),
        ),
        migrations.AlterField(
            model_name="post",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"post_type": "page"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="_children",
                to="djpress.post",
            ),
        ),
    ]
